#' check if ccdb is healthy
#'
#' @param uid user id of cctb database
#' @param ccsubdb the db used to check
#' @param ccidtaxdb the taxdb
#' @param taxdb_raw  raw taxdb
#' @param taxdb_fixed fixed taxdb
#' @param fixit fix the database or not
#' @param updateccdb update ccdb
#' @param updatetaxdb updata taxdb
#' @param fn tempfile xlsx file name for checking problematic data
#'
#' @return NULL
#' @importFrom utils head tail
#' @importFrom stats df setNames
#' @importFrom cctdb get_query edit_db upd_tbdf get_dbconn
#' @importFrom dplyr select arrange bind_rows
#' @importFrom dfeR get_dfdiff set_dfcls
#' @export
#'
check_ccdb <- function(uid,
                       ccsubdb = cctdb::get_query("SELECT `id`, `tax`, `sub`, `tot` FROM `cc`;",
                       dbname = paste0(uid, "pub")),
                       ccidtaxdb = dplyr::select(ccsubdb, id, tax),
                       taxdb_raw = check_taxdb(uid = uid),
                       taxdb_fixed = taxdb_raw,
                       fixit = FALSE,
                       updateccdb = FALSE,
                       updatetaxdb = FALSE,
                       fn = tempfile(pattern = "cctax_fixing_", fileext = ".xlsx")) {
  if (is.null(uid)) {
    stop("\n✖ `uid` should be provided!\n")
  }
  ccsubdb <- check_ccerror(
    ccsubdb = ccsubdb,
    uid = uid,
    fixit = fixit,
    updateccdb = updateccdb,
    fn = fn)
  ccidtaxdb <- check_taxerror(ccidtaxdb,
    uid = uid,
    taxdb_raw = taxdb_raw,
    taxdb_fixed = taxdb_fixed,
    fixit = fixit,
    updateccdb = updateccdb,
    updatetaxdb = updatetaxdb,
    fn = fn)
  return(list(ccsubdb = ccsubdb, ccidtaxdb = ccidtaxdb))
}

check_ccerror <- function(ccsubdb,
                          uid,
                          fixit = FALSE,
                          updateccdb = FALSE,
                          fn = tempfile(pattern = "cctax_fixing_", fileext = ".xlsx")) {
  dbname <- paste0(uid, "pub")
  # check tot is 0
  # 需要删除
  deldf <- ccsubdb |>
    dplyr::filter(is.na(tot) | (tot <= 0)) |>
    dplyr::filter(sub == 0)
  if (nrow(deldf) > 0) {
    askstr <- readline(prompt = "\n!!Enter `cc` to DELETE rows that have:\n ((`tot` IS NULL) OR (`tot` = 0)) AND ((`sub` IS NULL) OR (`sub` = 0))\n\n>>>")
    if (askstr != "cc") {
      message("\n Not excuted!\n")
    } else {
      sqlstr <- "DELETE FROM `cc` WHERE ((`tot` IS NULL) OR (`tot` = 0)) AND ((`sub` IS NULL) OR (`sub` = 0));"
      cctdb::edit_db(
        dbname = dbname,
        sqlstr = sqlstr,
        dbdisconn = TRUE
      )
      if (isTRUE(fixit)) {
        ccsubdb <- ccsubdb |>
          dplyr::filter(!((is.na(tot) | (tot <= 0)) & (sub == 0)))
      }
      message("\n√ Rows with error `tot` values deleted!\n")
    }
  }
  # 需要手动修复更新
  fixdf <- ccsubdb |>
    dplyr::filter(is.na(tot) | (tot <= 0)) |>
    dplyr::filter(sub > 0)
  if (nrow(fixdf) > 0) {
    sqlstr <- "SELECT * FROM `cc` WHERE ((`tot` IS NULL) OR (`tot` = 0)) AND (`sub` > 0);"
    errccsubdb <- cctdb::get_query(sqlstr = sqlstr, dbname = dbname) |>
      dplyr::mutate(id = as.character(id))
    errcclist <- list(ccdb = errccsubdb)
    fixcclist <- fixing_ccdb(errcclist)
    if (isTRUE(fixit)) {
      ccsubdb <- ccsubdb |>
        dplyr::filter(!(id %in% fixcclist$ccdb$id)) |>
        dplyr::bind_rows(
          fixcclist$ccdb |> dplyr::select(names(ccsubdb))
        ) |>
        dplyr::arrange(id)
    }
    if (identical(errcclist, fixcclist)) {
      message("\nNo updates from the excel file!\n")
    } else {
      fixccsubdb <- fixcclist$ccdb
      dfdiff <- dfeR::get_dfdiff(fixccsubdb, errccsubdb, cols_keep = "id") |>
        dplyr::mutate(id = as.numeric(id))
      cctdb::upd_tbdf(dfdiff, dbname = dbname, tbname = "cc", excute = updateccdb)
    }
  }
  # 需要将tax 变为Unknown
  nataxdf <- ccsubdb |>
    dplyr::filter(is.na(tax)) |>
    dplyr::filter((sub > 0) | (tot > 0)) |>
    dplyr::mutate(tax = "Unknown sp.") |>
    dplyr::select(id, tax)
  if (isTRUE(fixit)) {
    ccsubdb <- ccsubdb |>
      dplyr::mutate(tax = ifelse((is.na(tax) & ((sub > 0) | (tot > 0))),
        "Unknown sp.", tax
      ))
  }
  if (nrow(nataxdf) > 0) {
    message("\n Found NAs of `tax` in `ccdb`!\n")
    cctdb::upd_tbdf(nataxdf, dbname = dbname, tbname = "cc", excute = updateccdb)
  }
  return(ccsubdb)
}

check_taxerror <- function(ccidtaxdb,
                           uid,
                           taxdb_raw,
                           taxdb_fixed = taxdb_raw,
                           fixit = FALSE,
                           updateccdb = FALSE,
                           updatetaxdb = FALSE,
                           fn = tempfile(pattern = "cctax_fixing_", fileext = ".xlsx")) {
  if (is.null(uid) | is.null(taxdb_raw)) {
    stop("\n✖ `uid` and `taxdb_raw` should be provided!\n")
  }
  if (!(("id" %in% names(ccidtaxdb)) & ("tax" %in% names(ccidtaxdb)))) {
    stop("\n✖ `ccidtaxdb` should have `id` and `tax` columns!\n")
  }
  dbname <- paste0(uid, "pub")
  phytolang <- get_phytolang(ccidtaxdb)
  taxcolname <- paste0("tax", phytolang)
  errdf <- ccidtaxdb |>
    dplyr::select(tax) |>
    unique() |>
    dplyr::left_join(taxdb_fixed, by = c("tax" = taxcolname)) |>
    dplyr::filter(is.na(id)) |>
    dplyr::select(tax)
  if (nrow(errdf) == 0) {
    message("\n√ 所有`ccdb`中`tax`均在`taxdb`有记录！\n")
    return(ccidtaxdb)
  }
  taxcolname <- paste0("tax", phytolang)
  # 初始化修复字典
  errdictdf <- tibble(tax = NULL, taxraw = NULL)
  # 备份原始tax值
  ccidtaxdb <- ccidtaxdb |> dplyr::mutate(taxraw = tax)
  # 通过id列恢复
  if (!identical(taxdb_raw, taxdb_fixed)) {
    message("\n基于`id`列修复`ccdb`中部分`tax`列...\n")
    fixbyiddf <- errdf |>
      dplyr::left_join(taxdb_raw, by = c("tax" = taxcolname)) |>
      dplyr::filter(!is.na(id)) |>
      dplyr::select(tax, id) |>
      dplyr::left_join(taxdb_fixed, by = "id") |>
      dplyr::rename(TAX = taxcolname) |>
      dplyr::filter(!is.na(TAX)) |> # TODO
      dplyr::select(TAX, tax) |>
      dplyr::rename_all(~ c("tax", "taxraw"))
    # 如果id列可用于恢复
    if (nrow(fixbyiddf) > 0) {
      message("\n如下`tax`可通过`id`列修复！\n")
      fixbyiddf |> print(n = 1000)
      errdictdf <- errdictdf |> dplyr::bind_rows(fixbyiddf)
      # 将tax列中名字恢复至tax
      # 先修复部分ccidtaxdb
      ccidtaxdb <- ccidtaxdb |>
        dplyr::mutate(tax = recode(tax, !!!setNames(fixbyiddf$tax, fixbyiddf$taxraw)))
    }
  }
  # 去除空格问题
  message("\n❗️ 基于空白字符等检查`ccdb`...\n")
  ccidtaxdb <- ccidtaxdb |>
    dplyr::mutate(tax = gsub("^\\s{1,}(\\S)", "\\1", tax)) |>
    dplyr::mutate(tax = gsub("^\\t{1,}(\\S)", "\\1", tax)) |>
    dplyr::mutate(tax = gsub("(\\S)\\s{1,}$", "\\1", tax)) |>
    dplyr::mutate(tax = gsub("\\s{2,}", " ", tax)) |>
    dplyr::mutate(tax = gsub(" {2,}", " ", tax)) |>
    dplyr::mutate(tax = gsub("\\t{2,}", " ", tax)) |>
    dplyr::mutate(tax = gsub(" sp$| Sp$| sP$| SP$", " sp.", tax))
  # 如果两个不相同，说明有改动
  fixbyblkdf <- ccidtaxdb |>
    dplyr::filter(tax != taxraw) |>
    dplyr::select(tax, taxraw) |>
    unique()
  if (nrow(fixbyblkdf) > 0) {
    message("\n如下`tax`可通过空白字符等修复！\n")
    fixbyblkdf |> print(n = 1000)
    errdictdf <- errdictdf |> dplyr::bind_rows(fixbyblkdf)
  }
  # 再次检查
  message("\n基于`taxdb`检查`ccdb`...\n")
  errdf.2 <- ccidtaxdb |>
    dplyr::select(tax) |>
    unique() |>
    dplyr::left_join(taxdb_fixed, by = c("tax" = taxcolname)) |>
    dplyr::filter(is.na(id)) |>
    dplyr::filter(!is.na(tax)) |>
    dplyr::select(tax) |>
    dplyr::mutate(taxraw = tax)
  if (nrow(errdf.2) > 0) {
    message("\n以下`cc`中`tax`在`taxdb`中无记录！\n")
    errccidtaxdb <- ccidtaxdb |> dplyr::filter(tax %in% errdf.2$taxraw)
    names(errdf.2)[1] <- paste0("tax", phytolang)
    idnow <- lubridate::now()
    errdf.2 <- errdf.2 |>
      dplyr::mutate(id = parse_number(format(idnow + row_number() * 300, format = "%Y%m%d%H%M%S")))
    if (phytolang == "en") {
      errdf.2 <- errdf.2 |>
        dplyr::mutate(genusen = stringr::str_remove_all(taxen, " .*$")) |>
        dplyr::left_join(taxdb_fixed |>
          dplyr::select(genusen, genuscn, phytaen, phytacn) |>
          unique(), by = "genusen") |>
        dplyr::mutate(speciesen = taxen)
    } else {
      errdf.2 <- errdf.2 |> dplyr::mutate(speciescn = taxcn)
    }
    errtaxdb <- taxdb_fixed |>
      dplyr::bind_rows(errdf.2) |>
      dplyr::select(taxraw, dplyr::everything())
    # 修复ccidtaxdb与taxdb
    message("\n请在excel中修复`taxdb`！\n")
    errcclist <- list(ccdb = errccidtaxdb, taxdb = errtaxdb)
    # fixbytaxdflist <- fixing_ccdb(errcclist = errcclist, fn = fn)
    # 整理修复后的taxdb，去除重复行
    itry <- 0
    taxdb_fixed.2 <- 1
    taxdb_fixed.2.1 <- 2
    force <- FALSE
    while (!(identical(taxdb_fixed.2.1, taxdb_fixed.2) | isTRUE(force))) {
      if (itry == 0) { # 过滤第一次
        success <- file.copy(fn, basename(fn))
        if (success) {
          message("\n❗️ `taxdb`仍然存在问题，请继续修复！\n上次修改记录文件备份至`", getwd(), "/", basename(fn), "`中，\n可将更改复制到`", fn, "`中继续修改！\n")
        } else {
          message("\n❗️ `taxdb`仍然存在问题，请继续在`", fn, "`中修改！\n")
        }
      }
      fixbytaxdflist <- fixing_ccdb(errcclist = errcclist, fn = fn)
      taxdb_fixed.2 <- fixbytaxdflist$taxdb |>
        dplyr::arrange(id) |>
        dplyr::select(names(taxdb_raw)) |>
        tidyr::nest(iddf = -c(
          taxen, taxcn, phytacn, phytaen,
          genuscn, genusen, speciescn, speciesen
        )) |>
        dplyr::mutate(iddf = purrr::map(iddf, ~ .x |>
        dplyr::arrange(id) |>
        head(1))) |>
        tidyr::unnest(iddf) |>
        dplyr::select(names(taxdb_raw))
      # 检查taxdb
      message("检查", ifelse(updatetaxdb, "、修复并上传", ""), "`taxdb`...")
      taxdb_fixed.2.1 <- check_taxdb(uid = uid, taxdb = taxdb_fixed.2)
      itry <- itry + 1
      if (itry == 3) {
        readstr <- readline(prompt = "\n❗️ Force to update `taxdb` now!\nEnter `taxdb` to perform:\n>>>")
        if (readstr == "taxdb") {
          force <- TRUE
        }
      }
    }
    taxdiff <- dfeR::get_dfdiff(set_dfcls(taxdb_fixed.2, taxdb_raw), taxdb_raw, direction = "both") |>
      dplyr::rename(taxonen = taxen) |>
      dplyr::rename(taxoncn = taxcn) |>
      dplyr::filter_at(c("id", "taxonen", "taxoncn", "phytacn", "phytaen", "genuscn", "genusen", "speciescn", "speciesen"), dplyr::all_vars(!is.na(.)))
    cctdb::upd_tbdf(taxdiff, dbname, tbname = "tax", excute = updatetaxdb)
    fixbytaxdf <- fixbytaxdflist$taxdb |>
      dplyr::select(taxraw, paste0("tax", phytolang)) |>
      dplyr::filter(!is.na(taxraw)) |>
      dplyr::filter(taxraw != .data[[paste0("tax", phytolang)]]) |>
      dplyr::select(.data[[paste0("tax", phytolang)]], taxraw) |>
      dplyr::rename_all(~ c("tax", "taxraw"))
    errdictdf <- errdictdf |> dplyr::bind_rows(fixbytaxdf)
  }
  # 合并各种修复记录
  if (nrow(errdictdf) == 0) {
    message("\n√ No `tax` errors found in `ccdb`!\n")
    return(ccidtaxdb |> dplyr::select(id, tax))
  }
  # 基于errdictdf重新计算修复后的ccidtaxdb
  if (isTRUE(updateccdb)) {
    message("\n修复并上传至`ccdb`...\n")
    conn <- cctdb::get_dbconn(paste0(uid, "pub"), writepermission = TRUE)
    errdictdf |>
      dplyr::mutate(sqlstr = paste0("UPDATE `cc` SET `tax` = '", tax, "' WHERE `tax` = '", taxraw, "';")) |>
      dplyr::pull(sqlstr) |>
      purrr::walk(~ cctdb::edit_db(
        dbname = paste0(uid, "pub"),
        sqlstr = .x,
        conn = conn,
        dbdisconn = FALSE
      ))
    DBI::dbDisconnect(conn)
    message("\n√ 已修正`cc`表中`tax`字段并已上传至`ccdb`中！\n")
  }
  ccidtaxdb <- ccidtaxdb |>
    dplyr::mutate(tax = recode(taxraw, !!!setNames(errdictdf$tax, errdictdf$taxraw)))
  return(dplyr::select(ccidtaxdb, id, tax))
}

fixing_ccdb <- function(errcclist,
                        fn = tempfile(fileext = ".xlsx"),
                        localmode = TRUE) {
  if (isTRUE(localmode)) {
    writexl::write_xlsx(errcclist, fn)
    system(paste0("open ", fn))
    readstr <- NA
    while (!(readstr %in% c("done", "stop"))) {
      readstr <- readline(prompt = "Please type `done` when fixing process finished and excel file has been saved in default place!\nType `stop` to terminate!\n>>>")
    }
    if (readstr == "stop") {
      warning("Fixing process was terminated by user!")
      return(errcclist)
    }
    if (readstr == "done") {
      fixcclist <- names(errcclist) |>
        purrr::map(~ readxl::read_xlsx(fn, sheet = .x) %>%
          dplyr::mutate_at(dplyr::vars(matches("^date$")), ~ as.Date(.))) |>
        `names<-`(names(errcclist))
      fixcclist$ccdb <- dfeR::set_dfcls(fixcclist$ccdb, errcclist$ccdb)
    }
    return(fixcclist)
  }
  # 在线模式
  return(errcclist)
}
