#' Check and Fix the Taxonomic Database (taxdb)
#'
#' This function checks the integrity of the taxonomic database (`taxdb`)
#' associated with a specific user ID (`uid`). It can detect and optionally
#' fix issues such as blanks, missing values, erroneous names, and duplicate
#' entries. If requested, it can update the `taxdb` in the user's public
#' database.
#'
#' @param uid Character. The user ID associated with the `cctdb` database.
#'   This parameter is mandatory.
#' @param taxdb Data frame. The taxonomic database to be checked. Defaults
#'   to the result of `get_taxdb(uid, cols = "*")`.
#' @param fixit Logical. Whether to attempt fixing issues found in `taxdb`.
#'   Defaults to `FALSE`.
#' @param updatetaxdb Logical. Whether to update the fixed `taxdb` in the
#'   database. Defaults to `FALSE`.
#' @param fn Character. The file name for storing fixes temporarily if
#'   `fixit` is `TRUE`. Defaults to a temporary file with the `.xlsx`
#'   extension.
#'
#' @return Data frame. The checked and possibly fixed `taxdb`.
#'
#' @importFrom dplyr rename
#' @importFrom cctdb make_db
#'
#' @export
#'
check_taxdb <- function(uid,
                        taxdb = get_taxdb(uid, cols = "*"),
                        fixit = FALSE,
                        updatetaxdb = FALSE,
                        fn = ifelse(isTRUE(fixit),
                          tempfile(fileext = ".xlsx"),
                          NULL
                        )) {
  # Check for the presence of uid
  if (is.null(uid)) {
    stop("\n✖ `uid` should be provided!\n")
  }

  # Ensure taxdb has 24 columns
  if (ncol(taxdb) != 24) {
    stop("All columns of `taxdb` should be provided!")
  }

  # Perform checks and fixes if requested
  taxdb_fixed <- taxdb |>
    check_blanks(cols = 2:11, fixit = fixit) |>
    check_missings(fixit = fixit, fn = fn) |>
    check_errnames(fixit = fixit) |>
    check_duplicates(fixit = fixit, fn = fn)

  # Update the taxdb in the database if fixes were made and update is requested
  if (!identical(taxdb, taxdb_fixed)) {
    if (isTRUE(updatetaxdb)) {
      taxdb_fixed |>
        dplyr::rename(taxonen = "taxen", taxoncn = "taxcn") |>
        cctdb::make_db(
          tbname = "tax",
          dbname = paste0(uid, "pub"),
          keepstructure = TRUE
        )
    }
  }

  return(taxdb_fixed)
}

#' Check and Fix Blank Character Issues in `taxdb`
#'
#' This function checks for and optionally fixes issues related to blank
#' characters in the specified columns of the taxonomic database (`taxdb`).
#' It detects leading, trailing, and multiple consecutive blank characters,
#' as well as specific patterns like " sp", which could cause issues in data
#' processing.
#'
#' @param taxdb Data frame. The taxonomic database to be checked.
#' @param cols Integer or character vector. Columns to check for blank
#'   character issues. Defaults to columns 2 through 11.
#' @param fixit Logical. Whether to automatically fix detected blank
#'   character issues. Defaults to `FALSE`.
#'
#' @return Data frame. The original or corrected `taxdb` if `fixit` is
#'   `TRUE`. A message is also printed to indicate if issues were found and
#'   fixed.
#'
#' @importFrom dplyr mutate filter across ends_with select
#'
#' @export
#'
#' @examples
#' taxdb <- data.frame(
#'   taxen = c("Homo ", "Pan", "Gorilla", " Pongo"),
#'   speciesen = c("sapiens", "troglodytes", "gorilla", "pygmaeus"),
#'   stringsAsFactors = FALSE
#' )
#' taxdb_fixed <- check_blanks(taxdb, fixit = TRUE)
check_blanks <- function(taxdb, cols = 2:11, fixit = FALSE) {
  errdf <- taxdb |>
    dplyr::mutate(dplyr::across(dplyr::ends_with("cn"), iconv)) |>
    dplyr::filter(dplyr::across(
      cols,
      \(x) grepl("\\s{1,}$|^\\s{1,}|\\s{2,}| sp$| Sp$| sP$| SP$", x)
    ))

  if (nrow(errdf) > 0) {
    message(
      "\n❗️ Blank character typo found in `taxdb`, which may be caused by:\n",
      "1) start with blank;\n",
      "2) end with blank;\n",
      "3) more than one blank character in the strings.\n",
      "Possible columns are:\n",
      paste(colnames(taxdb)[cols], collapse = ",")
    )
    errdf |>
      dplyr::select(cols) |>
      print(n = 1000)
    if (isTRUE(fixit)) {
      taxdb <- taxdb |>
        dplyr::mutate(dplyr::across(dplyr::ends_with("cn"), iconv)) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("^\\s{1,}(\\S)", "\\1", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("(\\S)\\s{1,}$", "\\1", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("\\s{2,}", " ", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("\\s", " ", x))) |>
        dplyr::mutate(dplyr::across(
          c("taxen", "speciesen"),
          \(x) gsub(" sp$| Sp$| sP$| SP$", " sp.", x)
        ))

      message("\n√ `blank` problems fixed in `taxdb`!\n")
    }
  } else {
    message("\n√ `taxdb` doesn't have blank errors for specified columns!\n")
  }
  return(taxdb)
}

#' Check for and Fix Blank Character Issues in `taxdb`
#'
#' This function checks for and optionally fixes issues related to blank
#' characters in the specified columns of the taxonomic database (`taxdb`).
#' It detects leading, trailing, and multiple consecutive blank characters,
#' as well as specific patterns like " sp", which could cause issues in data
#' processing.
#'
#' @param taxdb Data frame. The taxonomic database to be checked.
#' @param cols Integer or character vector. Columns to check for blank
#'   character issues. Defaults to columns 2 through 11.
#' @param fixit Logical. Whether to automatically fix detected blank
#'   character issues. Defaults to `FALSE`.
#'
#' @return Data frame. The original or corrected `taxdb` if `fixit` is
#'   `TRUE`. A message is also printed to indicate if issues were found and
#'   fixed.
#'
#' @importFrom dplyr mutate filter across ends_with select
#'
#' @export
#'
#' @examples
#' taxdb <- data.frame(
#'   taxen = c("Homo ", "Pan", "Gorilla", " Pongo"),
#'   speciesen = c("sapiens", "troglodytes", "gorilla", "pygmaeus"),
#'   stringsAsFactors = FALSE
#' )
#' taxdb_fixed <- check_blanks(taxdb, fixit = TRUE)
check_blanks <- function(taxdb, cols = 2:11, fixit = FALSE) {
  errdf <- taxdb |>
    dplyr::mutate(dplyr::across(dplyr::ends_with("cn"), iconv)) |>
    dplyr::filter(dplyr::across(
      cols,
      \(x) grepl("\\s{1,}$|^\\s{1,}|\\s{2,}| sp$| Sp$| sP$| SP$", x)
    ))

  if (nrow(errdf) > 0) {
    message(
      "\n❗️ Blank character typo found in `taxdb`, which may be caused by:\n",
      "1) start with blank;\n",
      "2) end with blank;\n",
      "3) more than one blank character in the strings.\n",
      "Possible columns are:\n",
      paste(colnames(taxdb)[cols], collapse = ",")
    )

    errdf |>
      dplyr::select(cols) |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      taxdb <- taxdb |>
        dplyr::mutate(dplyr::across(dplyr::ends_with("cn"), iconv)) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("^\\s{1,}(\\S)", "\\1", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("(\\S)\\s{1,}$", "\\1", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("\\s{2,}", " ", x))) |>
        dplyr::mutate(dplyr::across(cols, \(x) gsub("\\s", " ", x))) |>
        dplyr::mutate(dplyr::across(
          c("taxen", "speciesen"),
          \(x) gsub(" sp$| Sp$| sP$| SP$", " sp.", x)
        ))

      message("\n√ `blank` problems fixed in `taxdb`!\n")
    }
  } else {
    message("\n√ `taxdb` doesn't have blank errors for specified columns!\n")
  }
  return(taxdb)
}

#' Check for Missing Data in `taxdb`
#'
#' This function checks for missing values (NA) or blank entries in specified
#' columns of the taxonomic database (`taxdb`). If requested, it can also
#' prompt the user to fix the missing data in an Excel file.
#'
#' @param taxdb Data frame. The taxonomic database to be checked.
#' @param fixit Logical. Whether to automatically fix missing values by
#'   loading an updated Excel file. Defaults to `TRUE`.
#' @param fn Character. Temporary file name for fixing. If `fixit` is
#'   `TRUE`, a temporary file with `.xlsx` extension will be created.
#'
#' @return Data frame. The original or updated `taxdb` with missing values
#'   fixed if `fixit` is `TRUE`.
#'
#' @importFrom dplyr filter select arrange bind_rows
#'
#' @export
#'
#' @examples
#' taxdb <- data.frame(
#'   taxen = c("Homo sapiens", NA, "Gorilla", "Pongo"),
#'   taxcn = c("人类", NA, "大猩猩", "红猩猩"),
#'   speciesen = c("sapiens", "troglodytes", "gorilla", "pygmaeus"),
#'   speciescn = c("人类", NA, "大猩猩", "小猩猩"),
#'   stringsAsFactors = FALSE
#' )
#' taxdb_fixed <- check_missings(taxdb, fixit = TRUE)
check_missings <- function(taxdb, fixit = TRUE,
                           fn = ifelse(isTRUE(fixit), tempfile(fileext = ".xlsx"), NULL)) {
  errdf <- taxdb |>
    dplyr::filter(
      dplyr::across(
        c(
          "taxen", "taxcn", "speciesen", "speciescn",
          "genusen", "genuscn", "phytaen", "phytacn"
        ),
        \(x) is.na(x) | grepl("^\\s{1,}$", x)
      )
    ) |>
    arrange("taxcn")
  if (nrow(errdf) == 0) {
    message("\n√ No missing values found in `taxdb`!\n")
    return(taxdb)
  }

  message(
    "\n❗️ `taxdb`\n  Follow rows have missing values!\n",
    "  Please fix them in Excel and load the fixed Excel file afterwards!"
  )

  errdf |>
    dplyr::select(c(
      "taxcn", "taxen", "speciescn", "speciesen",
      "genuscn", "genusen", "phytacn", "phytaen"
    )) |>
    print(n = 10000)

  if (isFALSE(fixit)) {
    return(taxdb)
  }

  fixdf <- fixing_taxdb(errdf, fn = fn)
  taxdb <- taxdb |>
    dplyr::filter(!(.data$id %in% errdf$id)) |>
    dplyr::bind_rows(fixdf)

  message("\n√ `missing value` problems fixed according to `", basename(fn), "`!\n")
  return(taxdb)
}

#' Check for Error Names in `taxdb`
#'
#' This function checks for naming errors in the taxonomic database (`taxdb`),
#' including specific characters in names, capitalization issues, and
#' matching inconsistencies between genus and species names.
#'
#' @param taxdb Data frame. The taxonomic database to be checked.
#' @param fixit Logical. Whether to automatically fix naming errors in
#'   `taxdb`. Defaults to `FALSE`.
#'
#' @return Data frame. The original or updated `taxdb` with naming errors
#'   fixed if `fixit` is `TRUE`.
#'
#' @importFrom dplyr select mutate filter group_by count
#' @importFrom stringr str_detect
#' @export
#'
check_errnames <- function(taxdb, fixit = FALSE) {
  # Check for "属" in genus names
  errdf <- taxdb |>
    dplyr::filter(grepl("属$", .data$genuscn))

  if (nrow(errdf) > 0) {
    message("\n❗️ Following genuscn has `属` in its name!")
    errdf |>
      dplyr::select("genuscn") |>
      unique() |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      taxdb <- taxdb |>
        dplyr::mutate(genuscn = gsub("属$", "", .data$genuscn))
      message("\n√ `属` problem fixed!\n")
    }
  }

  # Check for "门" in phylum names
  errdf <- taxdb |>
    dplyr::filter(grepl("门", .data$phytacn))

  if (nrow(errdf) > 0) {
    message("\n❗️ Following phytacn has `门` in its name!")
    errdf |>
      dplyr::select("phytacn") |>
      unique() |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      taxdb <- taxdb |>
        dplyr::mutate(phytacn = gsub("门$", "", .data$phytacn))
      message("\n√ `门` problem fixed!\n")
    }
  }

  # Check for lowercase initials
  errdf <- taxdb |>
    dplyr::filter(dplyr::across(
      c("taxen", "speciesen", "genusen", "phytaen"),
      \(x) grepl("^[^A-Z]", x)
    ))

  if (nrow(errdf) > 0) {
    message("\n❗️ Following rows not started with capital characters on `taxen`, `speciesen`, `genusen`, `phytaen` name!")
    errdf |>
      dplyr::select(c("taxen", "speciesen", "genusen", "phytaen")) |>
      unique() |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      fixdf <- fixing_taxdb(errdf)
      taxdb <- taxdb |>
        dplyr::filter(!(.data$id %in% errdf$id)) |>
        dplyr::bind_rows(fixdf)
      message("\n√ `Uncapitalized` problems fixed!\n")
    }
  }

  # Check for spaces in taxen and speciesen
  errdf <- taxdb |>
    dplyr::filter(dplyr::across(
      c("taxen", "speciesen"),
      \(x) !grepl(" ", x)
    ))

  if (nrow(errdf) > 0) {
    message("\n❗️ taxen, speciesen names should contain spaces!")
    errdf |>
      dplyr::select("taxen", "speciesen") |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      fixdf <- fixing_taxdb(errdf)
      taxdb <- taxdb |>
        dplyr::filter(!(.data$id %in% errdf$id)) |>
        dplyr::bind_rows(fixdf)
      message("\n√ `taxen, speciesen names without spaces` problems fixed!\n")
    }
  }

  # Check if genus and species names match
  errdf <- taxdb |>
    dplyr::filter(gsub(" .*$", "", .data$speciesen) != .data$genusen)

  if (nrow(errdf) > 0) {
    message("\n❗️ genusen and speciesen names do not match!")
    errdf |>
      dplyr::select(c("genusen", "speciesen", "taxen")) |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      fixdf <- fixing_taxdb(errdf)
      taxdb <- taxdb |>
        dplyr::filter(!(.data$id %in% errdf$id)) |>
        dplyr::bind_rows(fixdf)
      message("\n√ `genusen and speciesen names do not match` problems fixed!\n")
    }
  }

  # Check if genuscn matches speciescn
  errdf <- taxdb |>
    dplyr::filter(!stringr::str_detect(.data$speciescn, .data$genuscn))

  if (nrow(errdf) > 0) {
    message("\n❗️ genuscn and speciescn names do not match!")
    errdf |>
      dplyr::select(c("genuscn", "speciescn", "taxcn")) |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      fixdf <- fixing_taxdb(errdf)
      taxdb <- taxdb |>
        dplyr::filter(!(.data$id %in% errdf$id)) |>
        dplyr::bind_rows(fixdf)
      message("\n√ `genuscn and speciescn names do not match` problems fixed!\n")
    }
  }

  # Check for duplicate genus names
  dupgenuscn <- taxdb |>
    dplyr::select(c("genuscn", "genusen", "phytacn", "phytaen")) |>
    unique() |>
    dplyr::group_by("genuscn") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  # Check for duplicate genus names in the English version
  dupgenusen <- taxdb |>
    dplyr::select(c("genuscn", "genusen", "phytacn", "phytaen")) |>
    unique() |>
    dplyr::group_by("genusen") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  errdf <- taxdb |>
    dplyr::filter((.data$genuscn %in% dupgenuscn$genuscn) |
      (.data$genusen %in% dupgenusen$genusen))

  if (nrow(errdf) > 0) {
    message("\n❗️ genuscn, genusen, phytacn, and phytaen are not uniquely corresponding!")
    errdf |>
      dplyr::select(c("genuscn", "genusen", "phytacn", "phytaen")) |>
      unique() |>
      dplyr::arrange("genuscn") |>
      print(n = 1000)

    if (isTRUE(fixit)) {
      fixdf <- fixing_taxdb(errdf)
      taxdb <- taxdb |>
        dplyr::filter(!(.data$id %in% errdf$id)) |>
        dplyr::bind_rows(fixdf)
      message("\n√ `genuscn, genusen, phytacn, and phytaen are not uniquely corresponding` problems fixed!\n")
    }
  }

  return(taxdb)
}

#' Check for Duplicated Records in `taxdb`
#'
#' This function checks for duplicated records in the taxonomic database (`taxdb`)
#' based on specified columns and provides an option to fix the duplicates.
#'
#' @param taxdb Data frame. The taxonomic database to be checked.
#' @param fixit Logical. Whether to automatically fix duplicated records in
#'   `taxdb`. Defaults to `TRUE`.
#' @param fn Character. The filename to save the fixed records if `fixit` is `TRUE`.
#'
#' @return Data frame. The original or updated `taxdb` with duplicates removed if
#'   `fixit` is `TRUE`.
#'
#' @importFrom dplyr filter group_by count select bind_rows arrange
#' @export
#'
check_duplicates <- function(taxdb, fixit = TRUE,
                             fn = ifelse(isTRUE(fixit),
                               tempfile(fileext = ".xlsx"), NULL
                             )) {
  nduptaxcn <- taxdb |>
    dplyr::group_by("taxcn") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  nduptaxen <- taxdb |>
    dplyr::group_by("taxen") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  ndupspcn <- taxdb |>
    dplyr::group_by("speciescn") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  ndupspen <- taxdb |>
    dplyr::group_by("speciesen") |>
    dplyr::count() |>
    dplyr::filter(n > 1)

  if (nrow(nduptaxcn) + nrow(nduptaxen) + nrow(ndupspcn) + nrow(ndupspen) == 0) {
    message("\n√ No duplicated records found in `taxdb`!\n")
    return(taxdb)
  }

  message("\n❗️ Follow rows have duplicated records!\n  Please fix them in Excel and load the fixed Excel file afterwards!")

  errdf <- taxdb |>
    dplyr::filter(
      (.data$taxcn %in% nduptaxcn$taxcn) |
        (.data$taxen %in% nduptaxen$taxen) |
        (.data$speciescn %in% ndupspcn$speciescn) |
        (.data$speciesen %in% ndupspen$speciesen)
    ) |>
    dplyr::arrange("taxcn")

  errdf |>
    dplyr::select(c(
      "taxcn", "taxen", "speciescn", "speciesen",
      "genuscn", "genusen", "phytacn", "phytaen"
    )) |>
    print(n = 10000)

  if (isFALSE(fixit)) {
    return(taxdb)
  }

  fixdf <- fixing_taxdb(errdf, fn = fn)
  taxdb <- taxdb |>
    dplyr::filter(!(.data$id %in% errdf$id)) |>
    dplyr::bind_rows(fixdf)

  message(
    "\n√ `duplicated records` problems fixed according to `",
    basename(fn), "`!\n"
  )
  return(taxdb)
}

#' Fixing Tax Database
#'
#' This function allows users to correct errors in the taxonomic database
#' (`errdf`) by exporting the errors to an Excel file for manual editing.
#'
#' @param errdf Data frame. The error dataframe containing records to be fixed.
#' @param fn Character. The filename for the Excel file where errors will be
#'   exported. Defaults to a temporary file.
#' @param sheet Character. The sheet name in the Excel file. Defaults to
#'   "taxdb".
#' @param localmode Logical. Whether to operate in local mode. Defaults to
#'   `TRUE`.
#'
#' @return Data frame. The fixed records after manual correction from the Excel
#'   file.
#' @importFrom readxl read_xlsx
#' @importFrom writexl write_xlsx
#' @export
#'
fixing_taxdb <- function(errdf, fn = tempfile(fileext = ".xlsx"),
                         sheet = "taxdb",
                         localmode = TRUE) {
  if (isTRUE(localmode)) {
    errdflist <- list(errdf)
    names(errdflist)[1] <- sheet
    writexl::write_xlsx(errdflist, fn)
    system(paste0("open ", fn))
    readstr <- NA
    while (!(readstr %in% c("done", "stop"))) {
      readstr <- readline(
        prompt =
          "Please type `done` when fixing process finished and the Excel file has been saved in the default place!\n
        Type `stop` to terminate!\n>>> "
      )
    }
    if (readstr == "stop") {
      warning("Fixing process was terminated by user!")
      return(errdf)
    }
    if (readstr == "done") {
      fixdf <- readxl::read_xlsx(fn, sheet = sheet)
    }
    return(fixdf)
  }
  # Online mode (not implemented in this snippet)
  return(invisible(NULL)) # Return NULL or appropriate object for online mode
}
