#' New Version, to Check
#'
#' @param uid Character. cctb uid.
#' @param errtaxfn Character. Optional filename for the error taxonomies.
#' @param execute Logical. If TRUE, execute the SQL commands.
#' @export
#'
fix_cctax <- function(uid, errtaxfn = NULL, execute = FALSE) {
  conn <- cctdb::get_dbconn(
    ifelse(grepl("ucc", uid), paste0(uid, "pub"), uid),
    writepermission = TRUE
  )
  if (!("cc" %in% DBI::dbListTables(conn))) {
    warning("No valid data table found!")
    return(invisible(NULL))
  }
  # If errtaxfn is NULL, create an Excel file for error correction
  if (is.null(errtaxfn)) {
    taxdb <- dplyr::tbl(conn, "tax") |>
      dplyr::collect()
    err <- tibble::tibble(tax_error = "", tax_revised = "")
    del <- taxdb |> dplyr::filter(FALSE)
    upd <- taxdb |> dplyr::filter(FALSE)
    errtaxfn <- tempfile(fileext = ".xlsx")
    writexl::write_xlsx(list(taxdb = taxdb, err = err, del = del, upd = upd),
      path = errtaxfn
    )
    message(
      "Please open '", errtaxfn,
      "', fix the relevant errors, and save the file (e.g., ~/errtax_revised.xlsx). ",
      "Then run `fix_cctax(uid, errtaxfn = \"~/errtax_revised.xlsx\")` to fix the database!"
    )
    return(errtaxfn)
  }
  # Read in error taxonomies
  errtaxdf <- list(
    errdf = readxl::read_xlsx(errtaxfn, sheet = "err") |>
      dplyr::rename_all(~ c("tax_error", "tax_revised")),
    deldf = readxl::read_xlsx(errtaxfn, sheet = "del"),
    adddf = readxl::read_xlsx(errtaxfn, sheet = "upd")
  )
  # Update cc based on errdf
  sqlstr1 <- errtaxdf$errdf |>
    dplyr::mutate(sqlstr = paste0(
      "UPDATE `cc` SET `tax` = '",
      .data$tax_revised,
      "' WHERE `tax` = '", .data$tax_error, "';"
    )) |>
    dplyr::pull(.data$sqlstr) |>
    paste(collapse = " ")
  # Prepare to add new records
  adddf <- errtaxdf$adddf |>
    dplyr::mutate(id = ifelse(is.na(.data$id),
      as.numeric(format(
        lubridate::now() + dplyr::row_number(),
        "%Y%m%d%H%M%S"
      )),
      .data$id
    ))
  sqlstr2 <- cctdb::get_ondupsqlstr(adddf, uniqueids = "id", tbname = "tax")
  # Delete tax records
  sqlstr3 <- errtaxdf$deldf |>
    dplyr::transmute(sqlstr = paste0(
      "DELETE FROM `tax` WHERE `id` = ",
      .data$id, ";"
    )) |>
    dplyr::pull(.data$sqlstr) |>
    paste(collapse = " ")
  sqlstr <- paste(sqlstr1, sqlstr2, sqlstr3, sep = " ")
  # Execute SQL commands if requested
  if (isTRUE(execute)) {
    tryCatch(
      {
        DBI::dbExecute(conn, sqlstr)
      },
      error = function(e) {
        DBI::dbDisconnect(conn)
        print(e)
      }
    )
  }
  return(sqlstr)
}


#' Fix the Incorrect Tax Name in `cc` Table
#'
#' @param taxerror Character. The incorrect tax name.
#' @param taxcorrect Character. The correct tax name.
#' @param uid Character. cctb uid.
#' @param conn Database connection object. Defaults to a connection obtained
#'   from `cctdb::get_dbconn`.
#' @param dbdisconn Logical. Whether to disconnect from the database after
#'   execution. Defaults to TRUE.
#'
#' @importFrom dplyr tbl filter select
#' @importFrom cctdb get_dbconn edit_db
#' @importFrom stringr str_c
#' @importFrom DBI dbListTables
#' @export
#' @return Logical. Returns TRUE if the update was successful.
#'
#' @examples
#' if (FALSE) {
#'   uid <- "ucc0013"
#'   taxerror <- "纤维栅藻"
#'   taxcorrect <- "栅藻"
#'   fix_cctax(taxerror, taxcorrect, uid = uid)
#' }
#'
fix_cctax_bak <- function(taxerror,
                          taxcorrect,
                          uid = NULL,
                          conn = cctdb::get_dbconn(
                            ifelse(grepl("ucc", uid),
                              stringr::str_c(uid, "pub"),
                              uid
                            ),
                            writepermission = TRUE
                          ),
                          dbdisconn = TRUE) {
  if (!("cc" %in% DBI::dbListTables(conn))) {
    warning("No valid data table found!")
    return(invisible(NULL))
  }

  # Fetch records with incorrect tax names
  incorrect_records <- dplyr::tbl(conn, "cc") |>
    dplyr::filter(tax == taxerror) |>
    dplyr::select(id:remark) |>
    dplyr::collect()

  if (nrow(incorrect_records) == 0) {
    message("No records found with the incorrect tax name.")
    return(invisible(NULL))
  }

  # Prompt user for confirmation before update
  sqlstr <- paste0(
    "UPDATE `cc` SET `tax` = '", taxcorrect,
    "' WHERE `tax` = '", taxerror, "';"
  )
  ans <- readline(paste0(
    "Type 'YES' to replace '",
    taxerror, "' by '", taxcorrect,
    "'!\nThe SQL string is: \n", sqlstr, "\n"
  ))
  if (ans != "YES") {
    return(invisible(NULL))
  }

  # Execute the update
  flag <- DBI::dbExecute(conn, sqlstr)

  # Check if the update was successful
  if (isTRUE(flag)) {
    message("Executed successfully!")
  }

  # Disconnect if needed
  if (isTRUE(dbdisconn)) {
    DBI::dbDisconnect(conn)
  }

  return(invisible(flag))
}
