#' read cell counting data from MYSQL database in RCEES
#'
#' read cell counting data from MYSQL database in RCEES, which is
#' generated by the Cell Counting Tool, mainly read the cell
#' counting values and taxononmy database, return a dataframe
#'
#' @param uid database User ID, provided by RCEES
#' @param daterange date range
#' @param datelim (deprecated!) Date range as a vector of two dates.
#' @param pids project ids
#' @param sites  sites
#' @param zlim water depth range of samples
#' @param reps repeat samples id
#' @param taxs taxonomy names
#' @param extracond extra condition statement
#' @param cols variables to dplyr::select
#' @param conn connection to database server
#' @param dbdisconn disconnect conn when finished
#' @importFrom DBI dbListTables dbGetQuery dbDisconnect
#' @importFrom dplyr select filter rename pull mutate ungroup
#' @importFrom tibble as_tibble
#' @importFrom cctdb get_dbconn
#' @importFrom lubridate today as_date
#'
#' @export
#'
#' @return return a data frame with counting data and tax info.
#'
get_ccdb <- function(uid,
                     daterange = lubridate::today() - c(1460, 0),
                     pids = "",
                     sites = "",
                     zlim = c(0, 30),
                     reps = "",
                     taxs = "",
                     extracond = "",
                     cols = c(
                       "pid", "date", "site", "depth",
                       "rep", "tax", "den", "bio"
                     ),
                     conn = cctdb::get_dbconn(gsub("^(ucc.*)", "\\1pub", uid)),
                     datelim = NULL,
                     dbdisconn = TRUE) {
  if (!is.null(datelim)) {
    warning("'datelim' is deprecated; using 'daterange' instead!")
    daterange <- lubridate::ymd(datelim)
  }
  if (!("cc" %in% DBI::dbListTables(conn))) {
    warning("No valid datatable found!")
    brk_dbconn(conn, dbdisconn)
    return(invisible(NULL))
  }
  daterange <- lubridate::ymd(daterange)
  if (length(daterange) == 1) {
    daterange <- c(daterange, lubridate::today())
  }
  condstr <- get_ccsqlstr(
    daterange = daterange,
    pids = pids,
    sites = sites,
    zlim = zlim,
    reps = reps,
    taxs = taxs,
    extracond = extracond
  )
  if (grepl("ucc", uid)) {
    if (identical(cols, "*")) {
      selectstr <- "SELECT * FROM `cc`"
    } else {
      cols[cols == "den"] <- "tot"
      cols <- DBI::dbGetQuery(conn, "DESC `cc`;") |>
        dplyr::filter(.data$Field %in% cols) |>
        dplyr::pull(.data$Field)
      selectstr <- paste0(
        "SELECT ",
        paste(paste0("`", cols, "`"), collapse = ", "),
        " FROM `cc`"
      )
    }
    sqlstr <- paste0(selectstr, condstr, ";")
    ccdb <- DBI::dbGetQuery(conn = conn, statement = sqlstr) |>
      dplyr::rename(den = "tot")
    brk_dbconn(conn, dbdisconn)
  } else if (grepl("pmws", uid)) {
    if (identical(cols, "*")) {
      selectstr <- "SELECT * FROM `cc`"
    } else {
      cols <- DBI::dbGetQuery(conn, "DESC `cc`;") |>
        dplyr::filter(.data$Field %in% cols) |>
        dplyr::pull(.data$Field)
      selectstr <- paste0(
        "SELECT ", paste(paste0("`", cols, "`"), collapse = ", "),
        " FROM `cc`"
      )
    }
    sqlstr <- paste0(selectstr, condstr, ";")
    ccdb <- DBI::dbGetQuery(conn = conn, statement = sqlstr)
    brk_dbconn(conn, dbdisconn)
  } else {
    brk_dbconn(conn, dbdisconn)
    stop("Database not supported!")
  }
  ccdb <- tibble::as_tibble(ccdb) |>
    dplyr::mutate(date = as.Date(.data$date)) |>
    dplyr::mutate(depth = as.numeric(.data$depth)) |>
    dplyr::mutate(den = ifelse(.data$den < 0, NA, .data$den)) |>
    dplyr::mutate(bio = ifelse(.data$bio < 0, NA, .data$bio)) |>
    dplyr::ungroup()
  message("cell density: 10000 cells/L")
  message("biovolume: uL/L")
  return(ccdb)
}
