#' Get Cell Counting Data Frame Using ccdb and taxdb
#'
#' This function merges and summarizes cell counting data from the cc database
#' and the tax database, returning a tibble data frame.
#'
#' @param ccdb A data frame from the cc database.
#' @param taxdb A data frame from the tax database.
#'
#' @return A tibble data frame containing summarized cell counting data.
#' @export
#'
#' @examples
#' ccdf <- get_ccdf(ccdb, taxdb)
#'
get_ccdf <- function(ccdb, taxdb, .names_check = FALSE) {
  taxdb <- taxdb |>
    dplyr::select("phytaen", "phytacn", "genuscn", "genusen", "taxcn", "taxen")
  ind_en <- grepl("^[A-z]", ccdb$tax)
  ccdb <- ccdb |>
    dplyr::mutate(tax = gsub("^\\s*", "", .data$tax))
  ccdf_en <- ccdb |>
    dplyr::filter(ind_en) |>
    dplyr::rename(taxen = .data$tax) |>
    dplyr::left_join(taxdb, by = "taxen") |>
    dplyr::select(
      c(
        "pid", "date", "site", "depth", "rep", "phytacn", "phytaen", "genuscn",
        "genusen", "taxcn", "taxen", "den", "bio"
      )
    )
  ccdf_cn <- ccdb |>
    dplyr::filter(!ind_en) |>
    dplyr::rename(taxcn = "tax") |>
    dplyr::left_join(taxdb, by = "taxcn") |>
    dplyr::select(
      c(
        "pid", "date", "site", "depth", "rep", "phytacn", "phytaen", "genuscn",
        "genusen", "taxcn", "taxen", "den", "bio"
      )
    )
  ccdf <- dplyr::bind_rows(ccdf_en, ccdf_cn) |>
    dplyr::summarize(
      den = sum(.data$den, na.rm = TRUE),
      bio = sum(.data$bio, na.rm = TRUE),
      .by = c(
        "pid", "date", "site", "depth", "rep", "phytacn",
        "phytaen", "genuscn", "genusen", "taxcn", "taxen"
      )
    )
  if (isTRUE(.names_check)) {
    ccdf$phytacn <- gsub("藻$", "藻门", ccdf$phytacn)
  } else {
    message("藻门名规则更新: 藻门名均以'门'结尾\n   当前默认不加, 建议通过.names_check = TRUE 启用新规则!")
  }
  return(ccdf)
}
