#' @export
#'
get_cellden <- function(
    ccdb,
    type = ccdb$type,
    ident = ccdb$ident,
    fuji = ccdb$fuji,
    wujing = ccdb$wujing,
    sqno = ccdb$sqno,
    totsqno = ccdb$totsqno,
    totsqvol = ccdb$sqvol,
    totfvno = ccdb$totfvno,
    totfvvol = ccdb$totfvvol,
    rulerlength = ccdb$rulerlength,
    sub = ccdb$sub,
    size = ccdb$size) {
  if (nrow(ccdb) > 1) {
    cellden <- sapply(seq_along(type),
      FUN = function(i) {
        get_cellden(ccdb[i, ])
      },
      simplify = TRUE,
      USE.NAMES = FALSE
    )
    return(cellden)
  }
  # 细胞、格子
  if (type == "cell" && ident == 1) {
    cellden <- get_cellden_bysq_forcelltype(fuji, totsqno, totsqvol, sqno, sub)
    return(cellden)
  }
  # 细胞、视野
  if (type == "cell" && ident == 2) {
    cellden <- get_cellden_byfv_forcelltype(
      fuji, totfvno, wujing,
      totfvvol, sqno, sub
    )
    return(cellden)
  }
  # 长度、格子
  if (type == "length" && ident == 1) {
    cellden <- get_cellden_bysq_forlengthtype(
      fuji, totsqno, wujing, totsqvol,
      sqno, sub, rulerlength, size
    )
    return(cellden)
  }
  # 长度、视野
  if (type == "length" && ident == 2) {
    cellden <- get_cellden_byfv_forlengthtype(
      fuji, totfvno, wujing, totfvvol,
      sqno, sub, rulerlength, size
    )
    return(cellden)
  }
  # 面积、格子
  if (type == "area" && ident == 1) {
    cellden <- get_cellden_bysq_forareatype(
      fuji, totsqno, wujing, totsqvol,
      sqno, sub, rulerlength, size
    )
    return(cellden)
  }
  # 面积、视野
  if (type == "area" && ident == 2) {
    cellden <- get_cellden_byfv_forareatype(
      fuji, totfvno, wujing, totfvvol,
      sqno, sub, rulerlength, size
    )
    return(cellden)
  }
  return(NULL)
}

#' @export
#'
get_cellvol <- function(cellden, vol) {
  return(cellden * abs(vol))
}

#' @export
#'
get_cellden_bysq_forcelltype <- function(fuji, totsqno, totsqvol, sqno, sub) {
  # 按细胞、格子计数 (万/L)
  cellden <- sub / sqno * totsqno / (totsqvol / 1e3) / fuji / 1e4
  return(cellden)
}

#' @export
#'
get_cellden_byfv_forcelltype <- function(fuji, totfvno, wujing, totfvvol, sqno,
                                         sub) {
  # 按细胞、视野计数 (万/L)
  # fv: field vision
  fvno <- sqno # 此时sqno为视野数
  wujingrate <- wujing / 10
  cellden <- sub / (fvno / wujingrate^2) *
    totfvno / (totfvvol / 1e3) / fuji / 1e4
  return(cellden)
}

#' @export
#'
get_cellden_bysq_forlengthtype <- function(fuji, totsqno, wujing,
                                           totsqvol, sqno,
                                           sub, rulerlength, size) {
  # 按长度、格子计数 (万/L)
  # rulerlength: 10x物镜下，测微尺单位长度对应的实际长度（μm）
  # 如Olympus BX51显微镜中的测微尺为方格形，20*20小格子组成的大方格边长在10倍物镜下，
  wujingrate <- wujing / 10
  celllength <- abs(size) # 此时size为细胞长度
  sub_cell <- (sub * (rulerlength / wujingrate)) / celllength # (cell)
  cellden <- sub_cell / sqno * totsqno / (totsqvol / 1e3) / fuji / 1e4
  return(cellden)
}

#' @export
#'
get_cellden_byfv_forlengthtype <- function(fuji, totfvno, wujing,
                                           totfvvol, sqno,
                                           sub, rulerlength, size) {
  # 按细胞、视野计数 (万/L)
  # fv: field vision
  fvno <- sqno # 此时sqno为视野数
  # rulerlength: 10x物镜下，测微尺单位长度对应的实际长度（μm）
  # 如Olympus BX51显微镜中的测微尺为方格形，20*20小格子组成的大方格边长在10倍物镜下，
  wujingrate <- wujing / 10
  celllength <- abs(size) # 此时size为细胞长度
  sub_cell <- (sub * (rulerlength / wujingrate)) / celllength # (cell)
  cellden <- sub_cell / (fvno / wujingrate^2) *
    totfvno / (totfvvol / 1e3) / fuji / 1e4
  return(cellden)
}

#' @export
#'
get_cellden_bysq_forareatype <- function(fuji, totsqno, wujing, totsqvol, sqno,
                                         sub, rulerlength, size) {
  # 按面积、格子计数 (万/L)
  # 此时，sub：sq中填入的数字为长x宽=面积（单位为测微尺单位长度^2）
  # rulerlength: 10x物镜下，测微尺单位长度对应的实际长度（μm）
  # 如Olympus BX51显微镜中的测微尺为方格形，20*20小格子组成的大方格边长在10倍物镜下，
  wujingrate <- wujing / 10
  cellarea <- abs(size) # 此时size为平均每个细胞占据的面积
  sub_cell <- (sub * (rulerlength / wujingrate)^2) / cellarea # (cell)
  cellden <- sub_cell / sqno * totsqno / (totsqvol / 1e3) / fuji / 1e4
  return(cellden)
}

#' @export
#'
get_cellden_byfv_forareatype <- function(fuji, totfvno, wujing, totfvvol, sqno,
                                         sub, rulerlength, size) {
  # 按面积、视野计数 (万/L)
  # fv: field vision
  fvno <- sqno # 此时sqno为视野数
  # rulerlength: 10x物镜下，测微尺单位长度对应的实际长度（μm）
  # 如Olympus BX51显微镜中的测微尺为方格形，20*20小格子组成的大方格边长在10倍物镜下，
  wujingrate <- wujing / 10
  cellarea <- abs(size) # 此时size为平均每个细胞占据的面积
  sub_cell <- (sub * (rulerlength / wujingrate)^2) / cellarea # (cell)
  cellden <- sub_cell / (fvno / wujingrate^2) *
    totfvno / (totfvvol / 1e3) / fuji / 1e4
  return(cellden)
}
