#' Get community data frame from taxdf or genusdf using different modes
#'
#' @param taxdf the data frame with tax or genus
#' @param modestr aggregation modes, "IN", "YMN", "SN", ...
#'
#' @return comdf tibble data frame
#' @importFrom dplyr select ungroup left_join summarize group_by group_by_at
#' @importFrom tidyr unnest pivot_wider
#' @importFrom stringr str_c
#' @export
#'
get_comdf <- function(taxdf, modestr = "IN") {
  if (grepl("P$", modestr)) {
    warning(
      "Phyta should not be a group variable, do you mean: '",
      gsub("P$", "N", modestr), "'?"
    )
  }
  unnestdf <- taxdf |>
    dplyr::ungroup() |>
    get_nestdf(modestr = modestr) |>
    tidyr::unnest("Vdf") |>
    dplyr::select(-"phyta")
  iddf <- unnestdf |>
    dplyr::select(1:(ncol(unnestdf) - 2)) |>
    unique()
  comdf <- unnestdf |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("modestr", "taxlevel")
    ) |>
    dplyr::left_join(iddf, by = "modestr") |>
    dplyr::select(names(unnestdf)) |>
    tidyr::pivot_wider(
      names_from = "taxlevel",
      values_from = "V",
      values_fill = 0
    ) |>
    dplyr::select(1:(ncol(unnestdf) - 2), get_taxorder(unnestdf))
  grpind <- (which(names(comdf) == "modestr") - 1)
  if (grpind > 0) {
    comdf <- comdf |> dplyr::group_by_at(1:grpind)
  }
  return(comdf)
}
