#' Calculate Dominant Index
#'
#' This function calculates the dominant index for a given dataframe.
#'
#' @param Vdf A dataframe containing the data.
#' @param vartouse A string specifying the column name to use. Default is the
#' last column name of the dataframe.
#' @param byphyta A boolean indicating whether to process by phyta.
#' Default is FALSE.
#'
#' @return A dataframe with the dominant index calculated.
#' @examples
#' # Example usage:
#' Vdf <- data.frame(taxlevel = c("species1", "species2"), V = c(10, 20))
#' result <- get_domindex(Vdf, byphyta = TRUE)
#' @export
#' @importFrom dplyr mutate arrange
#' @importFrom tidyr nest unnest
#' @importFrom purrr map
get_domindex <- function(Vdf,
                         vartouse = tail(names(Vdf), 1),
                         byphyta = FALSE) {
  if (isTRUE(byphyta)) { # Calculate group by phyta
    domindexdf <- Vdf |>
      tidyr::nest(Vdf = -1) |>
      dplyr::mutate(Vdf = purrr::map(
        Vdf,
        ~ .x |>
          dplyr::mutate(domind = .data$V / sum(.x$V, na.rm = TRUE)) |>
          dplyr::arrange(dplyr::desc(domind))
      )) |>
      tidyr::unnest(cols = Vdf)
    return(domindexdf)
  }
  domindexdf <- Vdf |>
    dplyr::mutate(domind = .data$V / sum(Vdf$V, na.rm = TRUE)) |>
    dplyr::arrange(dplyr::desc(.data$domind))
  return(domindexdf)
}


#' Calculate the Dominating Index
#'
#' This function calculates the dominating index for a given dataframe,
#' optionally grouping by phyta.
#'
#' @param Vdf A dataframe containing community data with a variable column 'V'.
#' @param vartouse A string specifying the column name to use for calculations.
#' Default is the last column of the dataframe.
#' @param byphyta A boolean indicating whether to process the data by phyta.
#' Default is FALSE.
#'
#' @return A dataframe with the dominating index calculated.
#' @examples
#' # Example usage:
#' Vdf <- data.frame(taxlevel = c("species1", "species2"), V = c(10, 20))
#' result <- get_domindex(Vdf, byphyta = TRUE)
#' @export
#' @importFrom dplyr mutate arrange
#' @importFrom tidyr nest unnest
#' @importFrom purrr map
get_domindex <- function(Vdf,
                         vartouse = tail(names(Vdf), 1),
                         byphyta = FALSE) {
  # Check if vartouse exists in Vdf
  if (!vartouse %in% names(Vdf)) {
    stop("The specified variable 'vartouse' does not exist in the dataframe.")
  }
  if (isTRUE(byphyta)) { # Calculate group by phyta
    domindexdf <- Vdf |>
      tidyr::nest(Vdf = -1) |>
      dplyr::mutate(Vdf = purrr::map(
        .data$Vdf,
        \(Vdf) Vdf |>
          dplyr::mutate(domind = .data[[vartouse]] / sum(Vdf[[vartouse]],
            na.rm = TRUE
          )) |>
          dplyr::arrange(dplyr::desc(.data$domind))
      )) |>
      tidyr::unnest(cols = Vdf)

    return(domindexdf)
  }

  # Calculate overall dominating index
  domindexdf <- Vdf |>
    dplyr::mutate(domind = .data[[vartouse]] / sum(Vdf[[vartouse]],
      na.rm = TRUE
    )) |>
    dplyr::arrange(dplyr::desc(.data$domind))

  return(domindexdf)
}
