#' Get Dominating Taxa with Different Aggregation Modes
#'
#' This function calculates the dominating taxa based on
#' specified aggregation modes.
#'
#' @param taxdf A data frame containing taxonomic data or genus information.
#' @param modestr Aggregation modes. Options include "IN", "YMN", "SN", "P",
#' "IP", "MP", etc.
#'
#' @return A data frame containing the dominating taxa with calculated indices.
#' @importFrom dplyr select rename left_join mutate filter group_vars group_by_at
#' @importFrom tidyr unnest
#' @export
#'
get_domindexdf <- function(taxdf, modestr = "IN", ...) {
  # Validate input for modestr
  # Add more valid modes as needed
  valid_modes <- c("N", "IN", "YMN", "YWN", "DN", "WN", "MN", "SN", "P", "IP", "YMP", "YWP", "MP", "WP", "DP")
  if (!modestr %in% valid_modes) {
    stop(
      "Invalid aggregation mode. Please choose from: ",
      paste(valid_modes, collapse = ", ")
    )
  }

  # Handle the case for phytalevel
  if (grepl("P$", modestr)) {
    den_df <- get_ecoindexdf(taxdf,
      ecoindex = FALSE,
      modestr = modestr, vartouse = "den", ...
    )
    bio_df <- get_ecoindexdf(taxdf,
      ecoindex = FALSE,
      modestr = modestr, vartouse = "bio", ...
    )

    # Rename columns for clarity
    dendomdf <- den_df |>
      dplyr::select("modestr", "totV") |>
      dplyr::rename(den = "totV")

    biodomdf <- bio_df |>
      dplyr::select("modestr", "totV") |>
      dplyr::rename(bio = "totV")

    # Process for non-phytalevel
    modestr1 <- gsub("P$", "N", modestr)
    df3 <- get_ecoindexdf(taxdf,
      ecoindex = FALSE,
      modestr = modestr1, vartouse = "den", ...
    ) |>
      dplyr::select("modestr", "totV")

    df4 <- get_ecoindexdf(taxdf,
      ecoindex = FALSE,
      modestr = modestr1, vartouse = "bio", ...
    ) |>
      dplyr::select("modestr", "totV")

    # Calculate dominance
    if (modestr == "P") {
      df5 <- dendomdf |>
        dplyr::mutate(totV = df3$totV) |>
        dplyr::mutate(domind_den = .data$den / .data$totV) |>
        dplyr::select(-"totV")

      df6 <- biodomdf |>
        dplyr::mutate(totV = df4$totV) |>
        dplyr::mutate(domind_bio = .data$bio / .data$totV) |>
        dplyr::select(-"totV")
    } else {
      df5 <- dendomdf |>
        dplyr::left_join(df3, by = "modestr") |>
        dplyr::mutate(domind_den = .data$den / .data$totV) |>
        dplyr::select(-"totV")

      df6 <- biodomdf |>
        dplyr::left_join(df4, by = "modestr") |>
        dplyr::mutate(domind_bio = .data$bio / .data$totV) |>
        dplyr::select(-"totV")
    }

    # Combine results
    df7 <- df5 |> dplyr::left_join(df6)
    grps <- unique(c(dplyr::group_vars(df7), "modestr", "phyta"))
    domindexdf <- df7 |> dplyr::group_by_at(grps)

    return(domindexdf)
  }

  # Process for non-phytalevel cases
  den_df <- get_ecoindexdf(taxdf,
    ecoindex = FALSE,
    modestr = modestr, vartouse = "den", ...
  )
  bio_df <- get_ecoindexdf(taxdf,
    ecoindex = FALSE,
    modestr = modestr, vartouse = "bio", ...
  )

  dendomdf <- den_df |>
    dplyr::select("modestr", "Vdf") |>
    tidyr::unnest("Vdf") |>
    dplyr::rename(den = "V", domind_den = "domind")

  biodomdf <- bio_df |>
    dplyr::select("modestr", "Vdf") |>
    tidyr::unnest("Vdf") |>
    dplyr::rename(bio = "V", domind_bio = "domind")

  # Combine the dominance data
  domdf <- dplyr::left_join(dendomdf, biodomdf)
  grps <- c(dplyr::group_vars(domdf), "modestr", "phyta", "taxlevel")
  domindexdf <- domdf |> dplyr::group_by_at(grps)
  return(domindexdf)
}
