#' Calculate Ecological Indices and Dominant Species Strings
#'
#' This function calculates various ecological indices and generates strings
#' representing the dominant species in a dataset.
#'
#' @param df A dataframe containing the data.
#' @param domP A numeric value indicating the threshold for dominant species.
#' Default is 0.1.
#' @param vartouse A string specifying the column name to use. Default is the
#' last column name of df.
#' @param ecoindex A boolean indicating whether to calculate ecological indices.
#' Default is TRUE.
#' @param lang A string specifying the language for the dominant species string.
#' Default is the result of get_phytolang(df).
#' @param domwithnumber A boolean indicating whether to include numbers in the
#' dominant species string. Default is FALSE.
#'
#' @export
#'
#' @importFrom dplyr last rename mutate
#' @importFrom purrr map_dbl map_chr
#' @importFrom vegan diversity rarefy fisher.alpha specnumber
#'
#' @return A dataframe with calculated indices and dominant species strings.
#'
#' @examples
#' # Example usage:
#' df <- data.frame(...) # Your data here
#' result <- get_ecoindex(df)
#'
get_ecoindex <- function(df,
                         domP = 0.1,
                         vartouse = tail(colnames(df), 1),
                         ecoindex = TRUE,
                         lang = get_phytolang(df),
                         domwithnumber = FALSE) {
  if (ecoindex) {
    df <- calculate_ecoindex(df, vartouse)
  } else {
    df <- calculate_species_richness(df, vartouse)
  }
  df <- generate_domstr(df, domP, domwithnumber, lang)
  return(df)
}

#' Calculate Ecological Indices
#'
#' This function calculates various ecological indices for a given dataframe.
#'
#' @param df A dataframe containing the data.
#' @param vartouse A string specifying the column name to use.
#'
#' @return A dataframe with calculated ecological indices.
#' @export
#' @importFrom dplyr rename mutate
#' @importFrom purrr map_dbl
#' @importFrom vegan diversity rarefy fisher.alpha specnumber
calculate_ecoindex <- function(df, vartouse) {
  df <- df |>
    dplyr::rename(Vdf = {{ vartouse }}) |>
    dplyr::mutate(
      totV = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) sum(Vdf$V, na.rm = TRUE)
      ),
      H = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) vegan::diversity(Vdf$V)
      ),
      simp = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) vegan::diversity(Vdf$V, "simpson")
      ),
      invsimp = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) vegan::diversity(Vdf$V, "inv")
      )
    ) |>
    dplyr::mutate(unbias.sim = purrr::map_dbl(
      .data$Vdf,
      \(Vdf) {
        if (sum(Vdf$V, na.rm = TRUE) == 0) {
          NA
        } else {
          vegan::simpson.unb(floor(1000 * Vdf$V / sum(Vdf$V)))
        }
      }
    )) |>
    dplyr::mutate(
      alpha = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) ifelse(
          grepl(
            "Error",
            try(vegan::fisher.alpha(floor(Vdf$V * 10000)), silent = TRUE)
          ),
          NA, vegan::fisher.alpha(floor(Vdf$V * 10000))
        )
      ),
      S = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) vegan::specnumber(Vdf$V)
      ),
      J = .data$H / log(.data$S)
    )
}

#' Calculate Species Richness
#'
#' This function calculates species richness for a given dataframe.
#'
#' @param df A dataframe containing the data.
#' @param vartouse A string specifying the column name to use.
#'
#' @return A dataframe with calculated species richness.
#'
#' @export
#'
#' @importFrom dplyr rename mutate
#' @importFrom purrr map_dbl
#' @importFrom vegan specnumber
#'
calculate_species_richness <- function(df, vartouse) {
  df |>
    dplyr::rename(Vdf = {{ vartouse }}) |>
    dplyr::mutate(
      totV = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) sum(Vdf$V, na.rm = TRUE)
      ),
      S = purrr::map_dbl(
        .data$Vdf,
        \(Vdf) vegan::specnumber(Vdf$V)
      )
    )
}

#' Generate Dominant Species Strings
#'
#' This function generates strings representing the dominant species in a
#' dataset.
#'
#' @param df A dataframe containing the data.
#' @param domP A numeric value indicating the threshold for dominant species.
#' @param domwithnumber A boolean indicating whether to include numbers in the dominant species string.
#' @param lang A string specifying the language for the dominant species string.
#'
#' @return A dataframe with dominant species strings.
#'
#' @export
#'
#' @importFrom dplyr mutate arrange desc filter pull
#' @importFrom purrr map_chr
#'
generate_domstr <- function(df,
                            domP,
                            domwithnumber,
                            lang) {
  if (domP >= 1) {
    df <- df |>
      dplyr::mutate(domstr = purrr::map_chr(
        .data$Vdf,
        \(Vdf) {
          Vdf <- Vdf |>
            dplyr::arrange(dplyr::desc(domind)) |>
            head(floor(domP))
          if (domwithnumber) {
            Vdf <- Vdf |>
              dplyr::mutate(domstr = paste0(taxlevel, "(", round(V, 2), ")"))
          } else {
            Vdf <- Vdf |>
              dplyr::mutate(domstr = taxlevel)
          }
          Vdf |>
            dplyr::pull(domstr) |>
            paste0(collapse = ifelse(lang == "cn", "，", ", "))
        }
      ))
    return(df)
  }
  df <- df |>
    dplyr::mutate(domstr = purrr::map_chr(.data$Vdf, ~ {
      .x <- .x |>
        dplyr::arrange(dplyr::desc(domind)) |>
        dplyr::filter(domind >= domP)
      if (domwithnumber) {
        .x <- .x |>
          dplyr::mutate(domstr = paste0(taxlevel, "(", round(V, 2), ")"))
      } else {
        .x <- .x |>
          dplyr::mutate(domstr = taxlevel)
      }
      .x |>
        dplyr::pull(domstr) |>
        paste0(collapse = ifelse(lang == "cn", "，", ", "))
    }))
  return(df)
}


#' 获取优势藻字符串，domP < 1时，取占比大于domP的藻种，当domP>1时，取前domP个藻种
#'
#' @param df data frame to calculate
#' @param domP dominating threshold
#' @param vartouse the variable going to calculate the index
#' @param ecoindex calculate the ecoindex or not
#' @param lang language to use
#' @param domwithnumber number of dominating tax per group
#'
#' @return
#' @importFrom dplyr rename mutate arrange pull filter  desc last
#' @importFrom vegan diversity fisher.alpha specnumber
#' @importFrom purrr map_dbl map_chr
#' @importFrom stringr str_c
#' @export
#'

# get_ecoindex <- function(df,
#                          domP = 0.1,
#                          vartouse = dplyr::last(colnames(df)),
#                          ecoindex = TRUE,
#                          lang = get_phytolang(df),
#                          domwithnumber = FALSE) {
#   if (ecoindex) {
#     df <- df |>
#       dplyr::rename(Vdf = {{ vartouse }}) |>
#       dplyr::mutate(totV = purrr::map_dbl(
#         .data$Vdf,
#         ~ sum(.x$V, na.rm = TRUE)
#       )) |>
#       dplyr::mutate(H = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::diversity(.x$V)
#       )) |>
#       dplyr::mutate(simp = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::diversity(.x$V, "simpson")
#       )) |>
#       dplyr::mutate(invsimp = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::diversity(.x$V, "inv")
#       )) |>
#       dplyr::mutate(unbias.sim = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::rarefy(floor(.x$V * 10000), 2) - 1
#       )) |>
#       dplyr::mutate(alpha = purrr::map_dbl(
#         .data$Vdf,
#         ~ ifelse(grepl(
#           "Error",
#           try(vegan::fisher.alpha(floor(.x$V * 10000)), silent = TRUE)
#         ), NA, vegan::fisher.alpha(floor(.x$V * 10000)))
#       )) |>
#       ## Species richness (S) and Pielou's evenness (J):
#       dplyr::mutate(S = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::specnumber(.x$V)
#       )) |>
#       dplyr::mutate(J = .data$H / log(.data$S))
#   } else {
#     df <- df |>
#       dplyr::rename(Vdf = {{ vartouse }}) |>
#       dplyr::mutate(totV = purrr::map_dbl(
#         .data$Vdf,
#         ~ sum(.x$V, na.rm = TRUE)
#       )) |>
#       dplyr::mutate(S = purrr::map_dbl(
#         .data$Vdf,
#         ~ vegan::specnumber(.x$V)
#       ))
#   }
#   if (domP >= 1) {
#     if (isTRUE(domwithnumber)) {
#       outdf <- df |>
#         dplyr::mutate(domstr = purrr::map_chr(
#           .data$Vdf,
#           ~ .x |>
#             dplyr::arrange(dplyr::desc(domind)) |>
#             head(floor(domP)) |>
#             dplyr::mutate(domstr = stringr::str_c(
#               taxlevel,
#               "(", round(V, 2), ")"
#             )) |>
#             dplyr::pull(domstr) |>
#             stringr::str_c(collapse = ifelse(lang == "cn", "，", ", "))
#         ))
#       return(outdf)
#     }
#     outdf <- df |>
#       dplyr::mutate(domstr = purrr::map_chr(
#         .data$Vdf,
#         ~ .x |>
#           dplyr::arrange(dplyr::desc(domind)) |>
#           head(floor(domP)) |>
#           dplyr::mutate(domstr = taxlevel) |>
#           dplyr::pull(domstr) |>
#           stringr::str_c(collapse = ifelse(lang == "cn", "，", ", "))
#       ))
#     return(outdf)
#   }
#   if (domwithnumber) {
#     ecoindex <- df |>
#       dplyr::mutate(domstr = purrr::map_chr(
#         .data$Vdf,
#         ~ .x |>
#           dplyr::arrange(dplyr::desc(domind)) |>
#           dplyr::filter(domind >= domP) |>
#           dplyr::mutate(domstr = stringr::str_c(
#             taxlevel,
#             "(", round(V, 2), ")"
#           )) |>
#           dplyr::pull(domstr) |>
#           stringr::str_c(collapse = ifelse(lang == "cn", "，", ", "))
#       ))
#     return(ecoindex)
#   }
#   ecoindex <- df |>
#     dplyr::mutate(domstr = purrr::map_chr(
#       .data$Vdf,
#       ~ .x |>
#         dplyr::arrange(dplyr::desc(domind)) |>
#         dplyr::filter(domind >= domP) |>
#         dplyr::mutate(domstr = taxlevel) |>
#         dplyr::pull(domstr) |>
#         stringr::str_c(collapse = ifelse(lang == "cn", "，", ", "))
#     ))
#   return(ecoindex)
# }
#
