#' Calculate Ecological Indices and Dominant Species Strings for Taxonomic Data
#'
#' This function calculates various ecological indices and generates strings
#' representing the dominant species for taxonomic data.
#'
#' @param taxdf A dataframe containing the taxonomic data.
#' @param modestr A string specifying the mode for nesting data.
#' Default is "N".
#' @param domP A numeric value indicating the threshold for dominant species.
#' Default is 0.1.
#' @param vartouse A string specifying the column name to use. Default is "den".
#' @param ecoindex A boolean indicating whether to calculate ecological indices.
#' Default is TRUE.
#' @param lang A string specifying the language for the dominant species string.
#' Default is the result of get_phytolang(taxdf).
#' @param domwithnumber A boolean indicating whether to include numbers in the
#' dominant species string. Default is FALSE.
#'
#' @return A dataframe with calculated indices and dominant species strings.
#'
#' @examples
#' # Example usage:
#' taxdf <- data.frame(...) # Your taxonomic data here
#' result <- get_ecoindexdf(taxdf)
#'
#' @export
#'
#' @importFrom dplyr mutate group_vars group_by_at
#' @importFrom purrr map
#' @importFrom tidyr unnest nest
#'
get_ecoindexdf <- function(taxdf,
                           modestr = "N",
                           domP = 0.1,
                           vartouse = "den",
                           ecoindex = TRUE,
                           lang = get_phytolang(taxdf),
                           domwithnumber = FALSE) {
  nestdf <- get_nestdf(taxdf, modestr = modestr, vartouse = vartouse)
  if (grepl("N$", modestr)) { # 不按照门分类计算
    ecoindexdf <- calculate_ecoindex_for_nesting(nestdf, domP, lang,
      ecoindex, domwithnumber,
      byphyta = FALSE
    )
    return(ecoindexdf)
  }
  ecoindexdf <- process_phyta_nesting(nestdf) |>
    calculate_ecoindex_for_nesting(domP, lang,
      ecoindex, domwithnumber,
      byphyta = TRUE
    )
  return(ecoindexdf)
}

#' Calculate Ecological Indices for Nesting
#'
#' This function calculates ecological indices for nested data.
#'
#' @param df A dataframe containing the nested data.
#' @param domP A numeric value indicating the threshold for dominant species.
#' @param lang A string specifying the language for the dominant species string.
#' @param ecoindex A boolean indicating whether to calculate ecological indices.
#' @param domwithnumber A boolean indicating whether to include numbers in the
#' dominant species string.
#' @param byphyta A boolean indicating whether to process by phyta.
#'
#' @return A dataframe with calculated indices and dominant species strings.
#' @importFrom dplyr mutate
#' @importFrom purrr map
#'
calculate_ecoindex_for_nesting <- function(df,
                                           domP,
                                           lang,
                                           ecoindex,
                                           domwithnumber,
                                           byphyta) {
  df <- df |>
    dplyr::mutate(Vdf = purrr::map(
      Vdf,
      \(Vdf) get_domindex(Vdf, byphyta = byphyta)
    )) |>
    get_ecoindex(
      domP = domP,
      lang = lang,
      ecoindex = ecoindex,
      domwithnumber = domwithnumber
    )
  if (byphyta) {
    grps <- c(dplyr::group_vars(df), "phyta")
    df <- df |> dplyr::group_by_at(grps)
  }
  return(df)
}

#' Process Phyta Nesting
#'
#' This function processes data for phyta nesting.
#'
#' @param df A dataframe containing the nested data.
#'
#' @return A dataframe with processed phyta nesting.
#' @importFrom dplyr mutate
#' @importFrom purrr map
#' @importFrom tidyr unnest nest
process_phyta_nesting <- function(df) {
  df <- df |>
    dplyr::mutate(Vdf = purrr::map(
      Vdf,
      ~ get_domindex(.x, byphyta = TRUE)
    )) |>
    tidyr::unnest(cols = .data$Vdf) |>
    tidyr::nest(Vdf = c("taxlevel", "V", "domind"))
  return(df)
}


# #' 根据不同模式，获取优势藻矩阵
# #'
# #' @param taxdf the data frame with tax or genus
# #' @param modestr aggregation modes, "IN", "YMN", "SN", ...
# #' @param domP dominating threshold
# #' @param vartouse the variable going to calculate the index
# #' @param ecoindex calculate the ecoindex or not
# #' @param lang language to use
# #' @param domwithnumber number of dominating tax per group
# #'
# #' @return
# #' @importFrom dplyr mutate group_by_at group_vars
# #' @importFrom tidyr unnest nest
# #' @importFrom purrr map
# #' @export
# #'
# get_ecoindexdf <- function(taxdf,
#                            modestr = "N",
#                            domP = 0.1,
#                            vartouse = "den",
#                            ecoindex = TRUE,
#                            lang = get_phytolang(taxdf),
#                            domwithnumber = FALSE) {
#   # modestr <- "N"|"P"|"DN"|"DP"|"IN"|"IP"|"DSN"|"DSP"|...
#   df2 <- get_nestdf(taxdf, modestr = modestr, vartouse = vartouse)
#   if (grepl("N$", modestr)) { # 不按照门分类计算
#     ecoindexdf <- df2 |>
#       dplyr::mutate(Vdf = purrr::map(
#         Vdf,
#         ~ get_domindex(.x, byphyta = FALSE)
#       )) |>
#       get_ecoindex(
#         domP = domP,
#         lang = lang,
#         ecoindex = ecoindex,
#         domwithnumber = domwithnumber
#       )
#     return(ecoindexdf)
#   }
#   df3 <- df2 |>
#     dplyr::mutate(Vdf = purrr::map(
#       Vdf,
#       ~ get_domindex(.x, byphyta = TRUE)
#     )) |>
#     tidyr::unnest(cols = .data$Vdf) |>
#     tidyr::nest(Vdf = c("taxlevel", "V", "domind"))
#   grps <- c(dplyr::group_vars(df3), "phyta")
#   ecoindexdf <- df3 |>
#     get_ecoindex(
#       domP = domP,
#       lang = lang,
#       ecoindex = ecoindex,
#       domwithnumber = domwithnumber
#     ) |>
#     dplyr::group_by_at(grps)
#   return(ecoindexdf)
# }
