#' Get the Specified Number of Dominating Taxa
#'
#' This function retrieves the specified number of dominating taxa based on
#' the given variable (density or biomass) from the dominating data frame.
#'
#' @param domindexdf A data frame with dominating data.
#' @param vartouse A string specifying the variable to use for calculations
#' ("den" or "bio").
#' @param ndom An integer specifying the number of dominating taxa to return.
#' Default is 100 if `.by` is NULL; otherwise, it defaults to 3.
#'
#' @return A vector containing the specified number of dominating
#' taxa.
#'
#' @importFrom dplyr ungroup summarize mutate pull filter slice_max
#'
#' @export
#'
get_doms <- function(domindexdf,
                     level = "taxlevel",
                     vartouse = "den",
                     ndom = `if`(level == "phyta", 3, 10),
                     .otherby = c("total", "phyta"),
                     lang = get_phytolang(domindexdf)) {
  .otherby <- .otherby[1]
  # Validate inputs
  if (!vartouse %in% c("den", "bio")) {
    stop("Invalid 'vartouse'. Must be either 'den' or 'bio'.")
  }
  if (!is.numeric(ndom) || ndom < 1) {
    stop("Invalid 'ndom'. Must be a positive integer.")
  }
  domtax <- domindexdf |>
    dplyr::ungroup() |>
    dplyr::summarize(
      V = sum(.data[[vartouse]], na.rm = TRUE),
      .by = {{ level }}
    ) |>
    dplyr::mutate(domindex = .data$V / sum(.data$V, na.rm = TRUE))
  domtax <- if (ndom > 1) {
    domtax |>
      dplyr::slice_max(order_by = .data$V, n = ndom) |>
      dplyr::pull({{ level }})
  } else {
    domtax |>
      dplyr::slice_max(order_by = .data$V, prop = ndom) |>
      dplyr::pull({{ level }})
  }
  if (lang == "cn") {
    if (any(grepl("藻门$", domtax))) {
      othername <- "其他藻门"
    } else {
      if (.otherby == "phyta") {
        othername <- domindexdf |>
          dplyr::filter(.data$taxlevel %in% domtax) |>
          dplyr::pull(.data$phyta) |>
          unique() |>
          gsub(pattern = "门$", replacement = "", x = _) |>
          gsub(pattern = "^", replacement = "其他", x = _) |>
          gsub(pattern = "^其他其他", replacement = "其他", x = _)
        othername <- c(othername, "其他藻")
      } else {
        othername <- "其他藻"
      }
    }
  } else {
    if (.otherby == "phyta") {
      othername <- domindexdf |>
        dplyr::filter(.data$taxlevel %in% domtax) |>
        dplyr::pull(.data$phyta) |>
        unique()
      othername <- unlist(lapply(othername, \(x) {
        gsub(
          pattern = "^(.)",
          replacement = paste0("Other", tolower(substr(x, 1, 1))),
          x
        )
      }))
    } else {
      othername <- "Others"
    }
  }
  taxlevs <- c(domtax, othername)
  return(taxlevs)
}

#' Get the Specified Number of Dominating Taxa
#'
#' This function retrieves the specified number of dominating taxa based on
#' the given variable (density or biomass) from the dominating data frame.
#'
#' @param domindexdf A data frame with dominating data.
#' @param vartouse A string specifying the variable to use for calculations
#' ("den" or "bio").
#' @param ndom An integer specifying the number of dominating taxa to return.
#' Default is 100 if `.by` is NULL; otherwise, it defaults to 3.
#' @param .by Optional grouping variable. If provided, the function will
#' summarize by this variable.
#'
#' @return A filtered data frame containing the specified number of dominating
#' taxa.
#'
#' @importFrom dplyr ungroup summarize mutate pull filter slice_max
#'
#' @export
#'
get_ndomdf <- function(domindexdf,
                       level = "taxlevel",
                       vartouse = "den",
                       ndom = `if`(level == "phyta", 3, 10),
                       .by = NULL,
                       .otherby = c("total", "phyta"),
                       lang = get_phytolang(domindexdf)) {
  .otherby <- .otherby[1]
  # Validate inputs
  if (!vartouse %in% c("den", "bio")) {
    stop("Invalid 'vartouse'. Must be either 'den' or 'bio'.")
  }
  if (!is.numeric(ndom) || ndom < 1) {
    stop("Invalid 'ndom'. Must be a positive integer.")
  }
  orderbyvar <- ifelse(ndom > 1, vartouse, paste0("domind_", vartouse))
  if (is.null(.by)) {
    taxlevs <- get_doms(domindexdf,
      level = level,
      vartouse = vartouse,
      ndom = ndom,
      .otherby = .otherby,
      lang = lang
    )
    if (level == "taxlevel") {
      if (.otherby == "phyta") {
        if (lang == "cn") {
          ndomdf <- domindexdf |>
            dplyr::mutate(taxlevel = ifelse(
              .data$taxlevel %in% taxlevs,
              .data$taxlevel,
              ifelse(paste0("其他", gsub("门$", "", .data$phyta)) %in% taxlevs,
                paste0("其他", gsub("门$", "", .data$phyta)),
                "其他藻"
              )
            ))
        } else {
          ndomdf <- domindexdf |>
            dplyr::mutate(taxlevel = ifelse(
              .data$taxlevel %in% taxlevs,
              .data$taxlevel,
              paste0("Other", tolower(.data$phyta))
            ))
        }
        ndomdf <- ndomdf |>
          dplyr::summarize(dplyr::across(tidyselect::where(is.numeric), sum)) |>
          dplyr::mutate(taxlevel = factor(.data$taxlevel, levels = taxlevs))
        return(ndomdf)
      }
      # if (.otherby == "total") {
      ndomdf <- domindexdf |>
        dplyr::mutate(taxlevel = ifelse(
          .data$taxlevel %in% taxlevs, .data$taxlevel,
          tail(taxlevs, 1)
        )) |>
        dplyr::summarize(dplyr::across(tidyselect::where(is.numeric), sum)) |>
        dplyr::mutate(taxlevel = factor(.data$taxlevel, levels = taxlevs))
      return(ndomdf)
      # }
    }
    # phyta
    grpvars <- dplyr::group_vars(domindexdf)
    grpvars <- grpvars[grpvars != "taxlevel"]
    ndomdf <- domindexdf |>
      dplyr::ungroup() |>
      dplyr::mutate(phyta = ifelse(
        .data$phyta %in% taxlevs, .data$phyta,
        tail(taxlevs, 1)
      )) |>
      dplyr::summarize(dplyr::across(tidyselect::where(is.numeric), sum),
        .by = {{ grpvars }}
      ) |>
      dplyr::mutate(phyta = factor(.data$phyta, levels = taxlevs))
    return(ndomdf)
  }
  # Grouped case, NOT FINISHED, TODO
  if (ndom > 1) {
    return(dplyr::slice_max(domindexdf,
      order_by = .data[[orderbyvar]], n = ndom
    ))
  }
  return(dplyr::slice_max(domindexdf,
    order_by = .data[[orderbyvar]], prop = ndom
  ))
}
