#' Calculate Nest Data Frame
#'
#' @param taxdf A data frame containing taxonomic or genus data.
#' @param modestr A character string specifying the aggregation mode. Acceptable values include "N", "IN", "YMN", "SN", etc.
#' @param vartouse A character string indicating the variable used to calculate the index.
#' @param nested A logical indicating whether to nest the data frame or not.
#' @param lang A character string specifying the language to use.
#'
#' @return A data frame containing the summarized nest data.
#' @importFrom dplyr rename group_by summarize mutate
#' @importFrom tidyr nest unnest
#' @importFrom stringr str_c
#' @importFrom tidyselect all_of
#' @export
#'
get_nestdf <- function(taxdf,
                       modestr = "N",
                       vartouse = "den",
                       nested = TRUE,
                       lang = get_phytolang(taxdf)) {
  taxlevel <- intersect(c("tax", "genus"), colnames(taxdf))[1]

  # Calculate means for repno, etc.
  taxsmpdf <- taxdf |>
    dplyr::rename(
      V = tidyselect::all_of(vartouse),
      taxlevel = tidyselect::any_of(taxlevel)
    ) |>
    dplyr::summarize(
      V = sum(.data$V, na.rm = TRUE),
      .by = c("date", "site", "depth", "phyta", "taxlevel")
    )

  # Mapping of modestr to corresponding functions
  mode_functions <- list(
    "IN" = get_nestdf_I,
    "IP" = get_nestdf_I,
    "DSN" = get_nestdf_DS,
    "DSP" = get_nestdf_DS,
    "DN" = get_nestdf_D,
    "DP" = get_nestdf_D,
    "YWSN" = get_nestdf_YWS,
    "YWSP" = get_nestdf_YWS,
    "YMSN" = get_nestdf_YMS,
    "YMSP" = get_nestdf_YMS,
    "JSN" = get_nestdf_JS,
    "JSP" = get_nestdf_JS,
    "WSN" = get_nestdf_WS,
    "WSP" = get_nestdf_WS,
    "MSN" = get_nestdf_MS,
    "MSP" = get_nestdf_MS,
    "YSN" = get_nestdf_YS,
    "YSP" = get_nestdf_YS,
    "YWN" = get_nestdf_YW,
    "YWP" = get_nestdf_YW,
    "YMN" = get_nestdf_YM,
    "YMP" = get_nestdf_YM,
    "JN" = get_nestdf_J,
    "JP" = get_nestdf_J,
    "WN" = get_nestdf_W,
    "WP" = get_nestdf_W,
    "MN" = get_nestdf_M,
    "MP" = get_nestdf_M,
    "YN" = get_nestdf_Y,
    "YP" = get_nestdf_Y,
    "SN" = get_nestdf_S,
    "SP" = get_nestdf_S,
    "N" = get_nestdf_N,
    "P" = get_nestdf_P
  )

  # Call the appropriate function based on modestr
  if (modestr %in% names(mode_functions)) {
    return(mode_functions[[modestr]](taxsmpdf, nested, lang))
  }

  stop("Invalid modestr: '", modestr, "'!")
}


#' Nest Data Frame
#'
#' This function nests a data frame based on the specified parameters.
#'
#' @param unnestdf A data frame to be nested.
#' @param nested A logical value indicating whether to nest the data frame.
#' @return A nested data frame if `nested` is TRUE; otherwise, returns the original data frame.
mk_nestdf <- function(unnestdf, nested) {
  if (isFALSE(nested)) {
    return(unnestdf)
  }
  return(tidyr::nest(unnestdf, Vdf = c("phyta", "taxlevel", "V")))
}

#' Summarize Data Frame by Date, Site, Phyta, and Tax Level
#'
#' This function summarizes a data frame by calculating the mean of 'V'
#' grouped by date, site, phyta, and tax level.
#'
#' @param df A data frame containing the data to be summarized.
#' @return A summarized data frame with the mean value of 'V'.
mk_dsptsumdf <- function(df) {
  dplyr::summarize(df,
    V = mean(.data$V, na.rm = TRUE),
    .by = c("date", "site", "phyta", "taxlevel")
  )
}

#' Summarize Data Frame by Date, Phyta, and Tax Level
#'
#' This function summarizes a data frame by calculating the mean of 'V'
#' grouped by date, phyta, and tax level.
#'
#' @param df A data frame containing the data to be summarized.
#' @return A summarized data frame with the mean value of 'V'.
mk_dptsumdf <- function(df) {
  dplyr::summarize(df,
    V = mean(.data$V, na.rm = TRUE),
    .by = c("date", "phyta", "taxlevel")
  )
}


#' Handles modestr values of "IN" and "IP"
#'
#' @export
#'
get_nestdf_I <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    dplyr::mutate(modestr = paste(format(.data$date, "%Y%m%d"),
      .data$site, .data$depth,
      sep = "_"
    ))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "DSN" and "DSP"
#'
#' @export
#'
get_nestdf_DS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(modestr = paste0(as.character(.data$date), "_", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "DN" and "DP"
#'
#' @export
#'
get_nestdf_D <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    mk_dptsumdf() |>
    dplyr::mutate(modestr = as.character(date))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YWSN" and "YWSP" (year, week, site)
#'
#' @export
#'
get_nestdf_YWS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      year = lubridate::year(.data$date),
      week = lubridate::week(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "week", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = ifelse(
      lang == "cn",
      paste0(.data$year, "年第", .data$week, "周_", .data$site),
      paste0("Week ", .data$week, " of ", .data$year, "_", .data$site)
    ))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YMSN" and "YMSP" (year, month, site)
#'
#' @export
#'
get_nestdf_YMS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      year = lubridate::year(.data$date),
      month = lubridate::month(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "month", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = ifelse(
      lang == "cn",
      paste0(.data$year, "年", sprintf("%02d", .data$month), "月_", .data$site),
      paste0(month.abb[.data$month], ", ", .data$year, "_", .data$site)
    ))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "JSN" and "JSP" (julian, site)
#'
#' @export
#'
get_nestdf_JS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(jday = lubridate::yday(.data$date)) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("jday", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0("Day ", .data$jday, "_", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "WSN" and "WSP" (week, site)
#'
#' @export
#'
get_nestdf_WS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      week = lubridate::week(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("week", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0("Week ", .data$week, "_", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "MSN" and "MSP" (month, site)
#'
#' @export
#'
get_nestdf_MS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      month = lubridate::month(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("month", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(month.abb[.data$month], "_", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YSN" and "YSP" (year, site)
#'
#' @export
#'
get_nestdf_YS <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      year = lubridate::year(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(.data$year, "_", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YWN" and "YWP" (year, week)
#'
#' @export
#'
get_nestdf_YW <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      year = lubridate::year(.data$date),
      week = lubridate::week(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "week", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(.data$year, "_Week_", .data$week))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YMN" and "YMP" (year, month)
#'
#' @export
#'
get_nestdf_YM <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(
      year = lubridate::year(.data$date),
      month = lubridate::month(.data$date)
    ) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "month", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(.data$year, "_", sprintf("%02d", .data$month)))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "JN" and "JP" (julian)
#'
#' @export
#'
get_nestdf_J <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(jday = lubridate::yday(.data$date)) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("jday", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0("Day ", .data$jday))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "WN" and "WP" (week)
#'
#' @export
#'
get_nestdf_W <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(week = lubridate::week(.data$date)) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("week", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0("Week ", .data$week))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "MN" and "MP" (month)
#'
#' @export
#'
get_nestdf_M <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(month = lubridate::month(.data$date)) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("month", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(month.abb[.data$month]))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "YN" and "YP" (year)
#'
#' @export
#'
get_nestdf_Y <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::mutate(year = lubridate::year(.data$date)) |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("year", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0(.data$year))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "SN" and "SP" (site)
#'
#' @export
#'
get_nestdf_S <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("site", "phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = paste0("Site ", .data$site))
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "N" and "P"
#'
#' @export
#'
get_nestdf_N <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::summarize(
      V = mean(.data$V, na.rm = TRUE),
      .by = c("phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = "Total")
  return(mk_nestdf(unnestdf, nested))
}

#' Handles modestr values of "P"
#'
#' @export
#'
get_nestdf_P <- function(taxsmpdf, nested, lang) {
  unnestdf <- taxsmpdf |>
    mk_dsptsumdf() |>
    dplyr::summarize(
      V = sum(.data$V, na.rm = TRUE),
      .by = c("phyta", "taxlevel")
    ) |>
    dplyr::mutate(modestr = "Total")
  return(mk_nestdf(unnestdf, nested))
}
