#' Get Colour Values for Ordered Phylum Names
#'
#' This function retrieves the colour values associated with a specified
#' order of phyla.
#'
#' @param phytaorder A character vector of ordered phylum names.
#' @param coldf A logical value indicating whether to return the full
#'  colour data frame. If TRUE, the function returns a list containing
#'  both the colour vector and the complete colour data frame.
#'
#' @return A named character vector of colour values for the specified
#'   phylum names. If `coldf` is TRUE, returns a list containing:
#'    - `col`: the named vector of colour values.
#'    - `coldf`: the complete colour data frame.
#'
#' @importFrom tidyr pivot_longer
#' @importFrom dplyr mutate select filter slice
#' @export
#'
#' @examples
#' get_phytaorder(phytadf) |> get_phytacol()
#'
get_phytacol <- function(phytaorder, coldf = FALSE, .names_check = FALSE) {
  # Reshape the colour data frame and filter by the ordered phyla
  phytaorder <- gsub("藻$", "藻门", phytaorder)
  phytacoldf <- const_phytacoldf |>
    tidyr::pivot_longer(
      cols = 1:2,
      names_to = "lang",
      values_to = "phyta"
    ) |>
    dplyr::mutate(lang = gsub("phyta", "", .data$lang)) |>
    dplyr::select("phyta", "lang", "col") |>
    dplyr::filter(.data$phyta %in% phytaorder) |>
    dplyr::slice(match(.data$phyta, phytaorder))

  # Extract the colour values and assign names
  phytacol <- phytacoldf$col
  if (any(grepl("门$", phytaorder))) {
    names(phytacol) <- phytacoldf$phyta
  } else {
    names(phytacol) <- gsub("门$", "", phytacoldf$phyta)
  }

  # Optionally return the complete colour data frame
  if (coldf) {
    return(list(col = phytacol, coldf = phytacoldf))
  }

  return(phytacol)
}
