#' Get Yearly Data Frame of Phyla Concentrations
#'
#' This function calculates the average concentrations of specified variable
#' grouped by year and month for each phylum, while handling missing values.
#'
#' @param phytadf A data frame containing at least the columns `date` and the
#'                 variable specified by `vartoplot`.
#' @param vartoplot A character string indicating the name of the variable to
#'                  plot. Default is "den".
#'
#' @return A data frame with average concentrations of each phylum by year and
#'         month, including percentage contributions to total concentrations.
#'
#' @export
#' @importFrom dplyr rename ungroup full_join mutate group_by summarize
#' @importFrom lubridate year month
#'
#' @examples
#' yearly_data <- get_phytagrid_yearlydf(phytadf, vartoplot = "den")
#'
get_phytagrid_yearlydf <- function(phytadf, vartoplot = "den") {
  # Get the order of phyla and the grid of phyla by date
  phytaorder <- get_phytaorder(phytadf)
  phytagrid <- get_phytagrid(phytadf)

  # Calculate yearly average concentrations and percentage contributions
  yearlydf <- phytadf |>
    dplyr::rename(conc = tidyselect::all_of(vartoplot)) |>
    dplyr::ungroup() |>
    dplyr::full_join(phytagrid, by = c("date", "phyta")) |>
    dplyr::mutate(conc = ifelse(is.na(.data$conc), 0, .data$conc)) |>
    dplyr::group_by(
      year = lubridate::year(.data$date),
      month = lubridate::month(.data$date),
      .data$phyta
    ) |>
    dplyr::summarize(
      conc = mean(.data$conc, na.rm = TRUE),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      tconc = sum(.data$conc, na.rm = TRUE),
      .by = c("year", "month")
    ) |>
    dplyr::mutate(
      concperc = 100 * .data$conc / .data$tconc,
      .by = c("month", "phyta")
    ) |>
    dplyr::mutate(concperc = ifelse(.data$phyta == " ", 100, .data$concperc)) |>
    dplyr::mutate(phyta = factor(.data$phyta,
      levels = rev(c(" ", phytaorder))
    )) |>
    dplyr::arrange(.data$year, .data$month, .data$phyta)
  return(yearlydf)
}
