#' Phyla Grid Data Frame
#'
#' This function generates a data frame summarizing average concentrations of
#' specified variables for each phylum, grouped by month. Missing values are
#' replaced with zeros.
#'
#' @param phytadf A data frame containing at least the columns `date` and the
#'                 variable specified by `vartoplot`.
#' @param vartoplot A character string indicating the name of the variable to
#'                  plot. Default is "den".
#'
#' @return A data frame with average concentrations for each phylum by month,
#'         including total concentrations and percentage contributions.
#'
#' @export
#' @importFrom dplyr rename ungroup full_join group_by mutate summarize arrange
#' @importFrom lubridate month
#'
#' @examples
#' phytagriddf <- get_phytagriddf(phytadf)
#'
get_phytagriddf <- function(phytadf, vartoplot = "den") {
  phytaorder <- get_phytaorder(phytadf)
  phytagrid <- get_phytagrid(phytadf)
  phytagriddf <- phytadf |>
    dplyr::rename(conc = tidyselect::all_of(vartoplot)) |>
    dplyr::ungroup() |>
    dplyr::full_join(phytagrid, by = c("date", "phyta")) |>
    dplyr::mutate(conc = ifelse(is.na(.data$conc), 0, .data$conc)) |>
    dplyr::group_by(month = lubridate::month(.data$date), .data$phyta) |>
    dplyr::summarize(
      conc = mean(.data$conc, na.rm = TRUE),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      tconc = sum(.data$conc, na.rm = TRUE),
      .by = "month"
    ) |>
    dplyr::mutate(
      concperc = 100 * .data$conc / .data$tconc,
      .by = c("month", "phyta")
    ) |>
    dplyr::mutate(
      concperc =
        ifelse(.data$phyta == " ", 100, .data$concperc)
    ) |>
    dplyr::mutate(phyta = factor(.data$phyta,
      levels = rev(c(" ", phytaorder))
    )) |>
    dplyr::arrange("month", "phyta") |>
    dplyr::mutate(year = 2018)
  return(phytagriddf)
}
