#' Get Ordered Phyta
#'
#' This function generates an ordered vector of phylum names based on
#' their total densities or volumes, depending on the available data in the input
#' data frame. Optionally, it can include an "Other" category for unspecified phyla.
#'
#' @param phytadf A data frame at the phylum level containing either a `den`
#'                 (density) or `V` (volume) column.
#' @param withother A logical indicating whether to include an unspecified phyla
#'                  category labeled as "Other" (or "其他" in Chinese). Default is
#'                  FALSE.
#' @param lang A character string specifying the language to use for the "Other"
#'             category. Defaults to the output of `get_phytolang(phytadf)`.
#'
#' @return A character vector of ordered phylum names. If `withother` is TRUE,
#'         includes an additional "Other" category.
#'
#' @export
#' @importFrom dplyr group_by summarize pull arrange
#'
#' @examples
#' ordered_phyta <- get_phytaorder(phytadf, withother = TRUE)
#'
get_phytaorder <- function(phytadf,
                           withother = FALSE,
                           lang = get_phytolang(phytadf)) {
  # Summarize total density or volume by phylum and order them
  if ("den" %in% colnames(phytadf)) {
    phytaorder <- phytadf |>
      dplyr::summarize(
        den = sum(.data$den, na.rm = TRUE),
        .by = "phyta"
      ) |>
      dplyr::arrange(-.data$den) |>
      dplyr::pull("phyta")
  } else if ("V" %in% colnames(phytadf)) {
    phytaorder <- phytadf |>
      dplyr::summarize(
        V = sum(.data$V, na.rm = TRUE),
        .by = "phyta"
      ) |>
      dplyr::arrange(-.data$V) |>
      dplyr::pull("phyta")
  } else {
    stop("Input data frame must contain either 'den' or 'V' column.")
  }

  # Include "Other" category if specified
  if (isTRUE(withother)) {
    if (lang != "cn") {
      phytaorder <- c(phytaorder, "Others")
    } else if (any(grepl("门$", phytaorder))) {
      phytaorder <- c(phytaorder, ifelse(lang == "cn", "其他藻门", "Others"))
    } else {
      phytaorder <- c(phytaorder, ifelse(lang == "cn", "其他藻", "Others"))
    }
  }
  return(phytaorder)
}
