#' Get Phytoplankton Data Frame
#'
#' This function extracts and formats phytoplankton data from a cell counting
#' data frame based on the specified language. It removes unnecessary columns
#' and renames relevant ones according to the selected language.
#'
#' @param ccdf A data frame containing cell counting data, which includes
#'              columns for phytoplankton information.
#' @param lang A character string specifying the language to use for column
#'             names. Defaults to the output of `get_phytolang(ccdf)`.
#'
#' @return A data frame containing the formatted phytoplankton data with
#'  relevant columns renamed according to the selected language.
#'
#' @export
#' @importFrom dplyr ungroup select rename
#'
#' @examples
#' phytoplankton_df <- get_phytodf(cellcount_data)
#'
get_phytodf <- function(ccdf,
                        lang = get_phytolang(ccdf)) {
  phytodf <- switch(lang,
    "cn" = { # case cn
      ccdf |>
        dplyr::select(-c("phytaen", "genusen", "taxen")) |>
        dplyr::rename(phyta = "phytacn", genus = "genuscn", tax = "taxcn")
    },
    "en" = { # case en
      ccdf |>
        dplyr::select(-c("phytacn", "genuscn", "taxcn")) |>
        dplyr::rename(phyta = "phytaen", genus = "genusen", tax = "taxen")
    },
    { # case other
      ccdf |>
        dplyr::select(-c("phytaen", "genusen", "taxen")) |>
        dplyr::rename(phyta = "phytacn", genus = "genuscn", tax = "taxcn")
    }
  )
  return(phytodf)
}
