#' Get Phytoplankton Name Data Frame
#'
#' This function retrieves a data frame containing phytoplankton names and their
#' corresponding taxonomic levels based on a specified taxonomic data frame and
#' mode.
#'
#' @param taxdf A data frame containing taxonomic data, including information on
#'               phytoplankton species and their respective volumes.
#' @param modestr A character string specifying the mode to use.
#' Defaults to "PGSEC".
#' @param kable A logical value indicating whether to format the output as a
#' kable. Defaults to FALSE.
#'
#' @return A data frame containing phytoplankton names and their associated
#'  taxonomic levels, sorted by total volume in descending order.
#'
#' @export
#' @importFrom dplyr arrange desc select
#' @importFrom tidyr unnest
#'
#' @examples
#' phytoname_df <- get_phytonamedf(taxonomic_data)
#'
get_phytonamedf <- function(taxdf,
                            modestr = "PGSEC",
                            kable = FALSE) {
  # Retrieve and format the phytoplankton name data frame
  phytonamedf <- get_ecoindexdf(taxdf,
    modestr = "P",
    ecoindex = FALSE
  ) |>
    dplyr::arrange(dplyr::desc(.data$totV)) |>
    dplyr::select("phyta", "Vdf") |>
    tidyr::unnest("Vdf") |>
    dplyr::select("phyta", "taxlevel")
  return(phytonamedf)
}
