#' Get Colour Values for Taxonomic Groups
#'
#' This function generates colour values for ordered taxonomic groups
#' based on the specified order. If requested, it can also return a data
#' frame containing the taxonomic names and their associated colours.
#'
#' @param taxorder A character vector of ordered taxonomic names.
#' @param coldf A logical indicating whether to return a data frame
#'               with additional information (default is FALSE).
#'
#' @return If `coldf` is FALSE, a named vector of colour values for the
#'         taxonomic groups. If `coldf` is TRUE, a tibble containing
#'         the taxonomic names, language identifier, and associated colours.
#'
#' @importFrom tibble tibble
#' @export
#'
#' @examples
#' tax_colours <- get_taxcol(taxorder)
#' tax_colours_df <- get_taxcol(taxorder, coldf = TRUE)
#'
get_taxcol <- function(taxorder, coldf = FALSE) {
  # Generate colours for each taxonomic group
  taxcol <- taxcolpalette()
  names(taxcol) <- taxorder

  # Assign "gray80" to the last colour if the last taxonomic
  # name is "Other" or "其他"
  if (tail(taxorder, 1) %in% c("Others", "其他藻")) {
    taxcol[length(taxcol)] <- "gray80"
  }

  # If coldf is TRUE, return a tibble with taxonomic names and colours
  if (coldf) {
    taxcol <- tibble::tibble(
      tax = taxorder,
      lang = ifelse(any(grepl("藻", taxorder)), "cn", "en"),
      col = taxcol
    )
  }

  return(taxcol)
}
