#' Read Cell Counting Data from MySQL Database in RCEES
#'
#' This function retrieves cell counting data from the MySQL database
#' at RCEES, specifically designed for use with the Cell Counting Tool.
#' It returns a data frame containing cell counting values and taxonomy
#' information.
#'
#' @param uid Database User ID, provided by RCEES.
#' @param taxs Taxonomy names to filter the results.
#' @param cols Variables to select from the database. Default includes
#'             common taxonomy fields.
#' @param conn Connection to the database server. A default connection
#'             is established using the provided User ID.
#' @param dbdisconn Logical indicating whether to disconnect from the
#'                   database when finished (default is TRUE).
#'
#' @importFrom cctdb get_dbconn
#' @importFrom stringr str_c
#' @importFrom DBI dbGetQuery dbDisconnect dbListTables
#' @importFrom dplyr filter pull rename
#' @importFrom tibble as_tibble
#' @export
#'
#' @return A data frame containing cell counting data and taxonomy information.
#'
#' @examples
#' tax_data <- get_taxdb(uid = "your_user_id", taxs = "some_taxa")
#'
get_taxdb <- function(
    uid,
    taxs = "",
    cols = c(
      "taxonen", "taxoncn", "phytaen", "phytacn",
      "genuscn", "genusen", "speciesen", "speciescn"
    ),
    conn = cctdb::get_dbconn(gsub("^(ucc.*)$", "\\1pub", uid)),
    dbdisconn = TRUE) {
  # Check if the 'tax' table exists in the database
  if (!("tax" %in% DBI::dbListTables(conn))) {
    brk_dbconn(conn, dbdisconn) # Disconnect if table not found
    warning("No valid datatable found!")
    return(invisible(NULL))
  }

  # Construct SQL query for filtering taxonomy
  taxstr <- taxsqlstr(taxs, tbname = "tax")
  condstr <- if (!is.null(taxstr)) {
    paste0(" WHERE ", taxstr)
  } else {
    ""
  }

  # Construct SQL query for selecting columns
  if (identical(cols, "*")) {
    selectstr <- "SELECT * FROM `tax`"
  } else {
    cols <- tibble::tibble(DBI::dbGetQuery(conn, "DESC `tax`;")) |>
      dplyr::filter(.data$Field %in% cols) |>
      dplyr::pull("Field")

    selectstr <- paste0(
      "SELECT ",
      paste(paste0("`", cols, "`"), collapse = ", "),
      " FROM `tax`"
    )
  }

  # Combine the select and condition strings into a final SQL query
  sqlstr <- paste0(selectstr, condstr, ";")

  # Execute the query and convert the result into a tibble
  taxdb <- DBI::dbGetQuery(conn = conn, statement = sqlstr) |>
    tibble::as_tibble() |>
    dplyr::rename(taxen = "taxonen", taxcn = "taxoncn")

  # Disconnect from the database
  brk_dbconn(conn, dbdisconn)

  return(taxdb)
}
