#' Get Taxonomy Data Frame
#'
#' This function processes a phytoplankton data frame to calculate the
#' total density and biomass of each taxonomic group, taking into account
#' the number of replicates for each sample.
#'
#' @param phytodf A data frame containing phytoplankton data, including
#'                 density (`den`) and biomass (`bio`) measurements.
#'
#' @return A data frame summarizing the total density and biomass for each
#'         taxonomic group across the specified grouping variables.
#' @importFrom dplyr ungroup group_by summarize mutate
#' @export
#'
#' @examples
#' tax_data <- get_taxdf(phytodf)
#'
get_taxdf <- function(phytodf) {
  # Ensure that the number of replicates is considered when calculating
  # density and biomass. This handles cases where some replicates may
  # have a count of zero.
  taxdf <- phytodf |>
    dplyr::ungroup() |>
    dplyr::mutate(
      nrep = length(unique(rep)), # Count unique replicates
      .by = c("date", "site", "depth") # Grouping by sample characteristics
    ) |>
    dplyr::summarize(
      den = sum(.data$den, na.rm = TRUE) / mean(.data$nrep, na.rm = TRUE),
      bio = sum(.data$bio, na.rm = TRUE) / mean(.data$nrep, na.rm = TRUE),
      .by = c("date", "site", "depth", "phyta", "genus", "tax")
    )
  return(taxdf)
}
