#' Order the Taxonomic Groups
#'
#' This function orders taxonomic groups based on their total density or volume,
#' depending on the available columns in the provided data frame. It can also
#' group unspecified taxa into an "Other" category if desired.
#'
#' @param taxdf A taxonomically aggregated data frame. It should contain
#'               either the columns `taxlevel`, `tax`, or `genus`.
#' @param withother Logical indicating whether to include unspecified taxa
#'                  under the label "Other" or "其他" (for Chinese).
#'
#' @return A character vector of ordered taxonomic group names.
#' @importFrom dplyr group_by summarize pull arrange desc ungroup
#' @export
#'
#' @examples
#' ordered_taxa <- get_taxorder(taxdf, withother = TRUE)
#'
get_taxorder <- function(taxdf, withother = FALSE) {
  if ("taxlevel" %in% colnames(taxdf)) {
    taxorder <- taxdf |>
      dplyr::summarize(totV = sum(.data$V, na.rm = TRUE), .by = "taxlevel") |>
      dplyr::arrange(dplyr::desc(.data$totV)) |>
      dplyr::pull(.data$taxlevel)
  } else if ("tax" %in% colnames(taxdf)) {
    taxorder <- taxdf |>
      dplyr::summarize(tden = sum(.data$den, na.rm = TRUE), .by = "tax") |>
      dplyr::arrange(-.data$tden) |>
      dplyr::pull(.data$tax)
  } else if ("genus" %in% colnames(taxdf)) {
    taxorder <- taxdf |>
      dplyr::summarize(tden = sum(.data$den, na.rm = TRUE), .by = "genus") |>
      dplyr::arrange(-.data$tden) |>
      dplyr::pull(.data$genus)
  } else {
    warning("No available column for ordering taxonomic groups!")
    return(NULL)
  }

  if (withother && !is.null(taxorder)) {
    lang <- get_phytolang(taxdf)
    other_label <- ifelse(lang == "cn", "其他藻", "Others")
    taxorder <- c(taxorder, other_label)
  }

  # Ensure the "Other" or "其他" category is at the end
  if ("Others" %in% taxorder || "其他藻" %in% taxorder) {
    return(rev(unique(rev(taxorder))))
  }

  return(taxorder)
}
