#' Generate SQL Statement for PIDs or Sites
#'
#' This function constructs a SQL WHERE clause to filter records based on specified PIDs or site identifiers.
#' It handles individual IDs, ranges (e.g., "1-5"), and sanitizes input to prevent SQL injection.
#'
#' @param pidsorsites A character vector of PIDs or site identifiers. Can include ranges (e.g., "1-5") and
#'                    must be formatted correctly (e.g., separated by commas).
#' @param type A string indicating the column name to filter on (e.g., "pid" or "site").
#'
#' @return A character string representing the SQL WHERE clause for the provided PIDs or sites.
#' @importFrom stringr str_split str_c
#' @export
#'
#' @examples
#' sql_condition <- pidsitesqlstr("1, 2, 3-5", "pid")
#'
pidsitesqlstr <- function(pidsorsites, type) {
  # Return NULL if no PIDs or sites are provided
  if (is.null(pidsorsites)) {
    return(invisible(NULL))
  }

  # Clean and format the input
  pids <- gsub("[^0-9A-z]$|\\s{2,}|'", "", pidsorsites)
  pids <- gsub("[;；，]", ",", pids)
  pids <- unlist(stringr::str_split(pids, ","))

  # Separate individual IDs and ranges
  pids1 <- pids[!grepl("-", pids)]
  pidsseq <- stringr::str_split(pids[grepl("-", pids)], "-")

  # Expand ranges into individual IDs
  pids2 <- lapply(seq_along(pidsseq), function(x) {
    ind <- (as.numeric(gsub("[A-z]", "", pidsseq[[x]])))
    inds <- seq(ind[1], ind[2])
    prestr <- gsub("[0-9]", "", pidsseq[[x]][1])
    paste0(prestr, stringr::str_pad(inds,
      width = nchar(pidsseq[[x]][1]) - nchar(prestr),
      side = "left", pad = "0"
    ))
  }) |> unlist()

  # Combine and remove duplicates
  pids <- unique(c(pids1, pids2))
  pids <- pids[!grepl("^\\s*$", pids)]

  # Return NULL if no valid PIDs or sites remain
  if (length(pids) == 0) {
    return(invisible(NULL))
  }

  # Remove extra whitespace and construct SQL condition
  pids <- gsub("\\s{1,}", "", pids)
  pids_sql <- paste0(
    "(",
    paste(paste0("(`", type, "` LIKE '%", pids, "%')"), collapse = " OR "),
    ")"
  )

  return(pids_sql)
}
