#' Plot Word Cloud with Taxonomic Data
#'
#' This function creates a word cloud visualization for taxonomic data
#' using specified weights from the provided taxonomic data frame.
#' It allows for aggregation and customization of the plot.
#'
#' @param taxdf A data frame containing taxonomic level data.
#' @param modestr A string indicating the aggregation mode (e.g., "DN"
#' for density, "P" for phylum).
#' @param vartouse A string specifying the variable to calculate the
#' weight for the word cloud.
#' @param ntaxP An integer; when aggregating by phylum, it merges
#' groups with fewer than this number of taxa.
#' @param lambda A numeric value used to adjust the size of words in
#' the word cloud.
#' @param maxsize A numeric value defining the maximum size of words
#' in the word cloud.
#' @param ncol An integer indicating the number of columns for
#' faceting.
#' @param shape A string specifying the shape of the word cloud.
#' @param rm_outside A logical value indicating whether to remove text
#' outside the plot area.
#' @param trans A string specifying the transformation to apply to the
#' size scale.
#' @param ... Additional parameters passed to `geom_text_wordcloud_area`.
#'
#' @return A ggplot object representing the word cloud.
#' @import ggplot2
#' @importFrom dplyr mutate filter select left_join ungroup group_by
#' @importFrom tibble tibble
#' @importFrom tidyr unnest nest
#' @importFrom purrr map_dbl
#' @importFrom stringr str_c
#' @importFrom ggwordcloud geom_text_wordcloud_area
#' @export
#'
#' @examples
#' plot_cloudtax(taxdf, modestr = "DN", vartouse = "den")
plot_cloudtax <- function(taxdf,
                          modestr = "DN",
                          vartouse = "den",
                          ntaxP = 10, # Min taxa count for merging by phylum
                          lambda = 0.5,
                          maxsize = 8,
                          ncol = 4,
                          shape = "circle",
                          rm_outside = TRUE,
                          trans = "identity",
                          ...) {
  # Determine color order and palette
  phytaorder <- get_phytaorder(taxdf)
  cols <- get_phytacol(phytaorder)

  # Simple word cloud mode
  if (modestr == "simple") {
    p <- taxdf |>
      dplyr::rename(taxlevel = tidyselect::all_of(c("tax", "genus"))[1]) |>
      ggplot2::ggplot(aes(
        label = .data$taxlevel,
        size = .data[[vartouse]]^lambda,
        colour = .data$phyta
      )) +
      ggwordcloud::geom_text_wordcloud_area(
        rm_outside = FALSE,
        shape = shape
      ) +
      scale_size_area(max_size = maxsize, trans = trans) +
      scale_colour_manual(values = cols) +
      theme_minimal() +
      theme(strip.text = element_text(size = 12))
    return(p)
  }

  # Nest the data frame for word cloud
  coldf <- tibble::tibble(
    phyta = names(cols),
    colour = as.vector(cols)
  )
  nestdf <- get_nestdf(taxdf, modestr, vartouse)

  if (modestr == "P") { # Merge taxa with low counts when aggregating
    clouddf <- nestdf |>
      dplyr::mutate(ntax = purrr::map_dbl(.data$Vdf, nrow)) |>
      dplyr::mutate(modestr = ifelse(.data$ntax < ntaxP,
        "其他藻", modestr
      )) |>
      dplyr::mutate(modestr = factor(modestr, levels = phytaorder)) |>
      dplyr::select(-"ntax") |>
      tidyr::unnest("Vdf") |>
      dplyr::left_join(coldf, by = "phyta") |>
      dplyr::ungroup() |>
      dplyr::mutate(phyta = factor(.data$phyta,
        levels = phytaorder
      )) |>
      dplyr::arrange(modestr, dplyr::desc(.data$V))
  } else {
    clouddf <- nestdf |>
      tidyr::unnest("Vdf") |>
      dplyr::left_join(coldf, by = "phyta") |>
      dplyr::ungroup() |>
      dplyr::mutate(phyta = factor(.data$phyta,
        levels = phytaorder
      )) |>
      dplyr::arrange(modestr, dplyr::desc(.data$V))
  }

  # Create the word cloud plot
  p <- clouddf |>
    ggplot2::ggplot(aes(
      label = .data$taxlevel,
      size = .data$V^lambda,
      colour = .data$phyta
    )) +
    ggwordcloud::geom_text_wordcloud_area(
      rm_outside = rm_outside,
      shape = shape, ...
    ) +
    ggplot2::scale_size_area(max_size = maxsize, trans = trans) +
    ggplot2::scale_colour_manual(values = cols) +
    ggplot2::theme_minimal() +
    ggplot2::theme(strip.text = ggplot2::element_text(size = 12))

  # Add faceting if needed
  if (modestr != "N") {
    nPanel <- length(unique(clouddf$modestr))
    if (nPanel > 20) {
      warning("Too many (", nPanel, ") facet panels!")
    }
    p <- p + ggplot2::facet_wrap(~modestr,
      ncol = ncol,
      scales = "fixed"
    )
  }

  return(p)
}
