#' Plot Phyta Cycle with Taxonomic Data
#'
#' @param taxdf Tax level data frame.
#' @param modestr Aggregated modes.
#' @param specialtaxs Special taxa to use.
#' @param specialtaxsname Special taxa name to use.
#' @param vartouse Variable to calculate the weight.
#' @param ncol Number of columns for subfigures.
#'
#' @import ggplot2
#' @importFrom dplyr slice bind_rows ungroup mutate group_by filter select n
#' @importFrom tidyr nest unnest pivot_longer pivot_wider
#' @importFrom purrr map map_dbl
#' @importFrom lubridate ymd year month
#' @export
#' @return A ggplot object representing the phyta cycle plot.
plot_phytacycle <- function(taxdf,
                            modestr = "DP",
                            specialtaxs = NULL,
                            specialtaxsname = NULL,
                            vartouse = "den",
                            ncol = 2) {
  phytaorder <- get_phytaorder(taxdf)
  cols <- get_phytacol(phytaorder)

  if (!is.null(specialtaxs)) {
    specialtaxdf <- create_special_taxa_df(
      taxdf, specialtaxs,
      specialtaxsname
    )
    taxdf <- taxdf |>
      dplyr::bind_rows(specialtaxdf)
    phytaorder <- c(specialtaxsname, phytaorder)
    spcols <- "purple"
    names(spcols) <- specialtaxsname
    cols <- c(cols, spcols)
  }

  phyta_data <- get_phyta_data(taxdf, modestr, vartouse)

  plot_data <- nest_plot_data(phyta_data)

  plots <- create_plots(plot_data, cols)

  nrow <- ceiling(length(plots) / ncol)
  combined_plot <- patchwork::wrap_plots(plots, ncol = ncol) +
    patchwork::plot_layout(nrow = nrow)

  return(combined_plot)
}

# Create a special taxa data frame
create_special_taxa_df <- function(taxdf, specialtaxs, specialtaxsname) {
  if ("tax" %in% names(taxdf)) {
    specialtaxdf <- taxdf |>
      dplyr::filter(.data$genus %in% specialtaxs |
        .data$tax %in% specialtaxs) |>
      dplyr::mutate(phyta = specialtaxsname)
  } else {
    specialtaxdf <- taxdf |>
      dplyr::filter(.data$genus %in% specialtaxs) |>
      dplyr::mutate(phyta = specialtaxsname)
  }
  return(specialtaxdf)
}

# Get phyta data based on the mode
get_phyta_data <- function(taxdf, modestr, vartouse) {
  switch(modestr,
    "DP" = get_ecoindexdf_DP(taxdf, vartouse),
    "JP" = get_ecoindexdf_JP(taxdf, vartouse),
    "WP" = get_ecoindexdf_WP(taxdf, vartouse),
    "MP" = get_ecoindexdf_MP(taxdf, vartouse),
    "YWP" = get_ecoindexdf_YWP(taxdf, vartouse),
    "YMP" = get_ecoindexdf_YMP(taxdf, vartouse),
    "DSP" = get_ecoindexdf_DSP(taxdf, vartouse),
    "MSP" = get_ecoindexdf_MSP(taxdf, vartouse),
    "WSP" = get_ecoindexdf_WSP(taxdf, vartouse),
    stop("Unknown modestr value.")
  )
}

#' Get Ecoindex Data Frame for DP
#'
#' @export
get_ecoindexdf_DP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "DP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = .data$phyta,
      values_from = .data$totV, values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for JP
#'
#' @export
get_ecoindexdf_JP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "JP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::mutate(date = as.Date(.data$jday, origin = "1999-12-31")) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for WP
#'
#' @export
get_ecoindexdf_WP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "WP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::mutate(date = lubridate::ymd(20000101) +
      (.data$week - 1) * 7) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for MP
#'
#' @export
get_ecoindexdf_MP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "MP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::mutate(date = lubridate::ymd(paste(2000, month, 15, sep = "-"))) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for YWP
#'
#' @export
get_ecoindexdf_YWP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "YWP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::mutate(date = lubridate::ymd(paste(.data$year,
      "01-01",
      sep = "-"
    )) +
      (.data$week - 1) * 7) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for YMP
#'
#' @export
get_ecoindexdf_YMP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "YMP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::mutate(date = lubridate::ymd(paste(.data$year,
      .data$month, 15,
      sep = "-"
    ))) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for DSP
#'
#' @export
get_ecoindexdf_DSP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "DSP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for MSP
#'
#' @export
get_ecoindexdf_MSP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "MSP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

#' Get Ecoindex Data Frame for WSP
#'
#' @export
get_ecoindexdf_WSP <- function(taxdf, vartouse) {
  taxdf |>
    get_ecoindexdf(taxdf,
      modestr = "WSP",
      vartouse = vartouse, ecoindex = FALSE
    ) |>
    dplyr::ungroup() |>
    dplyr::select("date", "phyta", "totV") |>
    tidyr::pivot_wider(
      names_from = "phyta",
      values_from = "totV", values_fill = 0
    ) |>
    tidyr::pivot_longer(-.data$date,
      names_to = "phyta",
      values_to = "totV"
    )
}

# Nest plot data
nest_plot_data <- function(phyta_data) {
  phyta_data |>
    tidyr::nest(phydf = -c("date")) |>
    dplyr::arrange(.data$date)
}

# Create plots for each phyta
create_plots <- function(plot_data, cols) {
  plot_data |>
    dplyr::mutate(phyplot = purrr::map(
      .data$phydf,
      ~ ggplot2::ggplot(
        .x,
        ggplot2::aes(
          x = .data$date,
          y = .data$totV,
          fill = .data$phyta
        )
      ) +
        ggplot2::geom_area(position = "stack") +
        ggplot2::theme_classic() +
        ggplot2::scale_fill_manual(values = cols) +
        ggplot2::theme(
          axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
          legend.position = "right"
        ) +
        ggplot2::labs(
          title = unique(.x$phyta),
          x = NULL,
          y = NULL
        )
    )) |>
    dplyr::pull(phyplot)
}
