#' Plot Phyta Cycle by Yearly
#'
#' @param phytadf Phylum level data frame.
#' @param vartoplot Variable to plot.
#' @param ncol Number of columns for subfigures.
#'
#' @return A ggplot object representing the yearly phyta cycle plot.
#' @import ggplot2
#' @importFrom dplyr slice bind_rows ungroup mutate filter select n summarize group_by
#' @importFrom tidyr nest unnest pivot_longer pivot_wider
#' @importFrom purrr map map_dbl
#' @importFrom lubridate ymd year month
#' @export
#'
plot_phytayearlycycle <- function(phytadf,
                                  vartoplot = "den",
                                  ncol = 2) {
  lang <- get_phytolang(phytadf)
  phytagrid_yearlydf <- get_phytagrid_yearlydf(phytadf, vartoplot)
  phytaorder <- get_phytaorder(phytadf)
  cols <- get_phytacol(phytaorder)

  monthdf <- phytagrid_yearlydf |>
    dplyr::select(.data$year, .data$month) |>
    unique() |>
    dplyr::summarize(
      minmon = min(.data$month),
      maxmon = max(.data$month),
      .by = "year"
    )

  outdf <- adjust_months_for_years(phytagrid_yearlydf, monthdf)

  p <- create_yearly_cycle_plot(outdf, cols, lang, ncol)

  return(p)
}

# Adjust months for years in the data frame
adjust_months_for_years <- function(phytagrid_yearlydf, monthdf) {
  outdf <- phytagrid_yearlydf

  for (iyr in seq_along(monthdf$year)) {
    minM <- monthdf$minmon[iyr]
    maxM <- monthdf$maxmon[iyr]
    yr <- monthdf$year[iyr]

    if (minM == 1 && maxM == 12) {
      outdf <- add_january_data(outdf, yr)
    } else if (minM > 1 && maxM == 12) {
      outdf <- add_next_year_january_data(outdf, monthdf, yr)
    } else if (maxM < 12) {
      outdf <- add_missing_month_data(outdf, yr, maxM)
    }
  }

  return(outdf)
}

# Add January data for years where all months are present
add_january_data <- function(outdf, yr) {
  outdf |>
    dplyr::filter(year == yr, month == 1) |>
    dplyr::mutate(month = 13) |>
    dplyr::bind_rows(outdf)
}

# Add next year's January data if available
add_next_year_january_data <- function(outdf, monthdf, yr) {
  nextyrhasjan <- monthdf |>
    dplyr::filter(.data$year == yr + 1, .data$minmon == 1) |>
    nrow()

  if (nextyrhasjan) {
    outdf <- outdf |>
      dplyr::filter(year == yr + 1, month == 1) |>
      dplyr::mutate(year = yr, month = 13) |>
      dplyr::bind_rows(outdf)
  } else {
    outdf <- outdf |>
      dplyr::filter(year == yr, month == maxM) |>
      dplyr::mutate(month = 13, conc = NA, tconc = NA, concperc = NA) |>
      dplyr::bind_rows(outdf)
  }

  return(outdf)
}

# Add missing month data for the year
add_missing_month_data <- function(outdf, yr, maxM) {
  outdf |>
    dplyr::filter(year == yr, month == maxM) |>
    dplyr::mutate(month = 13, conc = NA, tconc = NA, concperc = NA) |>
    dplyr::bind_rows(outdf)
}

# Create the yearly cycle plot
create_yearly_cycle_plot <- function(outdf, cols, lang, ncol) {
  p <- outdf |>
    ggplot2::ggplot(aes(
      x = .data$month,
      y = .data$concperc,
      group = .data$phyta
    )) +
    ggplot2::geom_area(aes(fill = .data$phyta), position = "stack") +
    ggplot2::theme_light() +
    ggplot2::theme(legend.position = "top") +
    ggplot2::scale_fill_manual(values = cols) +
    ggplot2::coord_polar(theta = "x") +
    ggplot2::guides(fill = ggplot2::guide_legend(title = "")) +
    ggplot2::facet_wrap(~ factor(year), ncol = ncol, scales = "fixed") +
    ggplot2::theme(
      plot.background = ggplot2::element_rect(fill = "white"),
      panel.background = ggplot2::element_rect(colour = "black"),
      strip.text = ggplot2::element_text(size = 18)
    )

  if (lang == "cn") {
    p <- p + ggplot2::labs(x = NULL, y = "密度占比 [%]", fill = NULL) +
      ggplot2::scale_x_continuous(breaks = 1:12, labels = paste0(1:12, "月"))
  } else {
    p <- p + ggplot2::labs(x = NULL, y = "Cell density [%]", fill = NULL) +
      ggplot2::scale_x_continuous(breaks = 1:12, labels = month.abb)
  }

  return(p)
}
