#' Plot tax cycle
#'
#' @param taxdf A data frame at the taxonomic level.
#' @param modestr The mode of aggregation (e.g., "DP", "DN", etc.).
#' @param vartouse Variable to use for plotting.
#' @param specialtaxs Specific taxa to include in the plot (defaults to all).
#' @param taxorder The order of taxa for display.
#' @param cols Color palette for taxa.
#' @param lang Language for labels.
#' @param ncol Number of columns for subplot layout.
#'
#' @return A ggplot object representing the taxonomic cycle.
#' @import ggplot2
#' @importFrom dplyr slice bind_rows ungroup mutate filter select n summarize group_by
#' @importFrom tidyr nest unnest pivot_longer pivot_wider
#' @importFrom purrr map map_dbl
#' @importFrom lubridate ymd year month
#' @export
#'
#' @examples
#' plot_taxcycle(taxdf)
plot_taxcycle <- function(taxdf,
                          modestr = "DP",
                          vartouse = "den",
                          specialtaxs = NULL,
                          taxorder = get_taxorder(taxdf),
                          cols = get_taxcol(taxorder),
                          lang = get_phytolang(taxdf),
                          ncol = 2) {
  validate_modestr(modestr)
  taxdf <- prepare_taxdf(taxdf, lang, specialtaxs)

  # Warn if too many taxa
  if (length(taxorder) > 50) {
    warning("Too many taxa, unable to display!")
  }

  df1 <- generate_dataframe(taxdf, modestr)

  # Handle site or year classification
  df2 <- if ("site" %in% names(df1)) {
    process_site_data(df1)
  } else {
    process_year_data(df1)
  }

  # Plot
  create_plot(df2, cols, lang, ncol)
}

validate_modestr <- function(modestr) {
  valid_modes <- c(
    "DP", "DN", "JN", "WN", "MN",
    "YWN", "YMN", "DSN", "MSN", "WSN"
  )
  if (!(modestr %in% valid_modes)) {
    stop("Invalid modestr. Choose from: ", paste(valid_modes, collapse = ", "))
  }
}

prepare_taxdf <- function(taxdf, lang, specialtaxs) {
  if (!("tax" %in% names(taxdf))) {
    taxdf <- taxdf |>
      dplyr::rename(tax = "genus")
  }

  if (is.null(specialtaxs)) {
    specialtaxs <- unique(taxdf$tax)
  } else {
    specialtaxs <- unique(c(specialtaxs, ifelse(lang == "en", "Other", "其他")))
    taxdf <- taxdf |>
      dplyr::mutate(tax = ifelse(.data$tax %in% specialtaxs,
        .data$tax,
        ifelse(lang == "en", "Other", "其他")
      )) |>
      dplyr::mutate(phyta = ifelse(.data$tax %in% c("Other", "其他"),
        .data$tax, .data$phyta
      )) |>
      dplyr::summarize(
        den = sum(.data$den, na.rm = TRUE),
        bio = sum(.data$bio, na.rm = TRUE),
        .by = c("date", "site", "depth", "phyta", "tax")
      )
  }
  return(taxdf)
}

generate_dataframe <- function(taxdf, modestr) {
  switch(modestr,
    "DN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::select(-modestr) |>
      tidyr::pivot_longer(-date, names_to = "tax", values_to = "totV"),
    "JN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = as.Date(.data$jday, origin = "1999-12-31")) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "jday")) |>
      tidyr::pivot_longer(-date,
        names_to = "tax",
        values_to = "totV"
      ),
    "WN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(20000101) +
        (.data$week - 1) * 7) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "week")) |>
      tidyr::pivot_longer(-date, names_to = "tax", values_to = "totV"),
    "MN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(paste(2000, month, 15, sep = "-"))) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "month")) |>
      tidyr::pivot_longer(-date, names_to = "tax", values_to = "totV"),
    "YWN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(paste(.data$year, "01-01", sep = "-")) +
        (.data$week - 1) * 7) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "week", "year")) |>
      tidyr::pivot_longer(-date, names_to = "tax", values_to = "totV"),
    "YMN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(paste(year, month, 15, sep = "-"))) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "month", "year")) |>
      tidyr::pivot_longer(-date, names_to = "tax", values_to = "totV"),
    "DSN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::ungroup() |>
      dplyr::select(-modestr) |>
      tidyr::pivot_longer(-c("site", "date"),
        names_to = "tax", values_to = "totV"
      ),
    "MSN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(paste(2000, month, 15, sep = "-"))) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "month")) |>
      tidyr::pivot_longer(-c("site", "date"),
        names_to = "tax", values_to = "totV"
      ),
    "WSN" = get_comdf(taxdf, modestr = modestr) |>
      dplyr::mutate(date = lubridate::ymd(20000101) + (week - 1) * 7) |>
      dplyr::ungroup() |>
      dplyr::select(-c("modestr", "week")) |>
      tidyr::pivot_longer(-c("site", "date"),
        names_to = "tax", values_to = "totV"
      )
  )
}

process_site_data <- function(df1) {
  df1 |>
    tidyr::nest(taxdf = -c("date", "site")) |>
    dplyr::arrange("date") |>
    dplyr::group_by("site") |>
    tidyr::nest(ydf = -.data$site) |>
    dplyr::mutate(aa = purrr::map(.data$ydf, tmpfun1)) |>
    dplyr::select(-.data$ydf) |>
    tidyr::unnest("aa") |>
    dplyr::select(-c("mdf", "m")) |>
    tidyr::unnest("preddf") |>
    dplyr::ungroup() |>
    dplyr::select(c("site", "date", "tax", "totV")) |>
    dplyr::mutate(totV = ifelse(.data$totV < 0, 0, .data$totV)) |>
    dplyr::bind_rows(df1)
}

process_year_data <- function(df1) {
  df1 |>
    tidyr::nest(taxdf = -.data$date) |>
    dplyr::arrange(.data$date) |>
    dplyr::group_by(year = lubridate::year(.data$date)) |>
    tidyr::nest(ydf = -.data$year) |>
    dplyr::mutate(aa = purrr::map(.data$ydf, tmpfun1)) |>
    dplyr::select(-.data$ydf) |>
    tidyr::unnest("aa") |>
    dplyr::select(-c("mdf", "m")) |>
    tidyr::unnest("preddf") |>
    dplyr::ungroup() |>
    dplyr::select(c("year", "date", "tax", "totV")) |>
    dplyr::mutate(totV = ifelse(.data$totV < 0, 0, .data$totV)) |>
    dplyr::bind_rows(df1)
}

create_plot <- function(df2, cols, lang, ncol) {
  ggplot(df2, aes_string(x = "date", y = "totV", color = "tax")) +
    geom_line(size = 1) +
    scale_color_manual(values = cols) +
    labs(
      title = paste("Taxonomic Cycle -", lang),
      x = "Date",
      y = "Total Value"
    ) +
    facet_wrap(~tax, ncol = ncol) +
    theme_minimal() +
    theme(legend.position = "bottom")
}
