#' 获取cc数据表所有数据
#'
#' This function retrieves all data from the cc database table using a specified uid.
#'
#' @param uid A character string representing the ucc database ID.
#'
#' @return A data frame containing all records from the cc database.
#'
#' @export
get_ccdbraw <- function(uid) {
  ccdbraw <- get_ccdb(uid, daterange = 19900101, cols = "*")
  return(ccdbraw)
}

#' 检查并更新可能错误的sub数据列
#'
#' This function checks and updates potentially erroneous sub-data columns in the ccdbraw dataset.
#'
#' @param ccdbraw A data frame containing raw ccdb data retrieved from `get_ccdbraw(uid)`.
#'
#' @return A data frame with corrected sub-data columns and a new `sub` column representing the row sums.
#'
#' @importFrom dplyr mutate across
#'
#' @export
set_sub <- function(ccdbraw) {
  ccdbrevised <- ccdbraw |>
    dplyr::mutate(dplyr::across(.data$s1:.data$s100, as.numeric)) |>
    dplyr::mutate(sub = rowSums(dplyr::across(.data$s1:.data$s100),
      na.rm = TRUE
    )) |>
    dplyr::mutate(dplyr::across(
      .data$s1:.data$s100,
      \(x) ifelse(is.na(x), "", as.character(x))
    ))
  return(ccdbrevised)
}

#' 汇总不同样品的实际计数视野/格子数
#'
#' This function summarizes the actual count fields for different samples in the ccdbraw dataset.
#'
#' @param ccdbraw A data frame containing raw ccdb data retrieved from `get_ccdbraw(uid)`.
#'
#' @return A data frame summarizing the maximum count per sample and square number.
#'
#' @importFrom dplyr mutate group_by summarize filter ungroup
#' @importFrom tidyr pivot_longer
#'
#' @export
get_smpsqnodf <- function(ccdbraw) {
  if (is.null(ccdbraw)) {
    return(invisible(NULL))
  }

  smpsqnodf <- ccdbraw |>
    dplyr::mutate(dplyr::across(
      .data$s1:.data$s100,
      \(x) ifelse(is.na(x), "", x)
    )) |>
    dplyr::summarize(dplyr::across(.data$s1:.data$s100,
      \(x) paste(x, collapse = ""),
      .by = c("smp", "sqno")
    )) |>
    tidyr::pivot_longer(cols = 3:102, names_to = "sq", values_to = "sqdata") |>
    dplyr::filter(.data$sqdata != "") |>
    dplyr::mutate(sq = as.numeric(gsub("s", "", .data$sq))) |>
    dplyr::group_by(c("smp", "sqno")) |>
    dplyr::summarize(maxsq = max(.data$sq), .groups = "drop")

  return(smpsqnodf)
}

#' 检查并更新可能错误的sqno数据列
#'
#' This function checks and updates potentially erroneous sqno data columns in
#' the ccdb dataset.
#' Users are prompted to manually verify the data in an Excel file.
#'
#' @param ccdbraw A data frame containing ccdb data, should be the output of
#' `get_ccdbraw(uid) |> set_sub()`.
#' @param fn A character string specifying the filename for saving the Excel
#' file for manual verification.
#' @param smpsqnodf A data frame summarizing sqno data,
#' default is `get_smpsqnodf(ccdbraw)`.
#' @param savetoxlsx A logical indicating whether to save the data to an
#' Excel file (default is TRUE).
#'
#' @return A data frame containing the updated sqno data.
#'
#' @importFrom dplyr mutate filter left_join select
#' @importFrom readxl read_xlsx
#' @importFrom lubridate as_date
#'
#' @export
set_ccsqno <- function(
    ccdbraw = NULL,
    fn = tempfile(pattern = "cctax_fixing_", fileext = ".xlsx"),
    smpsqnodf = NULL,
    savetoxlsx = TRUE) {
  if (is.null(ccdbraw)) {
    stop("需要指定ccdbraw")
  }

  if (is.null(smpsqnodf)) {
    smpsqnodf <- get_smpsqnodf(ccdbraw)
  }

  smpsqnodf <- smpsqnodf |>
    dplyr::mutate(CHECK = ifelse(abs(.data$sqno - .data$maxsq) > 10,
      "MAY BE ERROR!", ""
    ))

  errorsmps <- smpsqnodf |>
    dplyr::filter(grepl("ERROR", .data$CHECK))

  if (nrow(errorsmps) > 0) {
    message("sqno needs to be checked manually for these samples!")
    print(errorsmps, n = 100000)
  }

  if (isTRUE(savetoxlsx)) {
    ccdbsqno <- ccdbraw |>
      dplyr::left_join(dplyr::select(smpsqnodf, -.data$sqno), by = "smp") |>
      dplyr::select(.data$id:.data$sqno, .data$maxsq, .data$CHECK, .data$totsqno:.data$s100)

    writexl::write_xlsx(list(sqno = ccdbsqno, raw = ccdbraw), fn)
    system(paste("open", fn))
  }

  readstr <- NA
  while (!(readstr %in% c("done", "stop"))) {
    readstr <- readline(
      prompt = "Please type `done` when fixing process finished and excel file has been saved in default place!\n
      Type `stop` to terminate!\n>>>"
    )
  }

  if (readstr == "stop") {
    warning("Fixing process was terminated by user!")
    return(errorsmps)
  }

  if (readstr == "done") {
    ccdbrevised <- readxl::read_xlsx(fn,
      sheet = "sqno",
      guess_max = nrow(ccdbraw)
    ) |>
      dplyr::select(colnames(ccdbraw)) |>
      dplyr::mutate(date = lubridate::as_date(date))

    return(ccdbrevised)
  }

  return(ccdbraw)
}

#' 检查并更新可能错误的细胞密度与生物量列
#'
#' This function checks and updates potentially erroneous cell density and
#' biomass columns in the ccdbraw dataset.
#'
#' @param ccdbraw A data frame containing raw ccdb data retrieved from
#' `get_ccdbraw(uid)`.
#'
#' @return A data frame with updated cell density (`den`) and biomass
#' '(`bio`) columns.
#'
#' @importFrom dplyr mutate
#'
#' @export
set_celldenbio <- function(ccdbraw) {
  ccdbrevised <- ccdbraw |>
    dplyr::mutate(
      den = get_cellden(ccdbraw),
      bio = get_cellvol(.data$den, abs(.data$vol))
    )
  return(ccdbrevised)
}

#' 将人工审核完的ccdb上传至数据
#'
#' This function uploads the manually reviewed ccdb data to the database
#' after confirming correctness.
#'
#' @param ccdbrevised A data frame containing the revised ccdb data after
#' manual checks.
#' @param uid A character string representing the ucc database ID.
#' @param conn A database connection object used for updating the database.
#' @param fn A character string specifying the filename for saving the Excel
#' file for confirmation.
#'
#' @return A logical indicating whether the data upload was successful.
#'
#' @importFrom dplyr mutate rename
#' @importFrom cctdb get_dbconn
#' @importFrom writexl write_xlsx
#' @importFrom DBI dbDisconnect dbExecute dbWriteTable
#'
#' @export
#'
#' @examples
#'
#' if (FALSE) {
#'   fn <- tempfile(pattern = "cctax_fixing_", fileext = ".xlsx")
#'   system(paste0("open ", fn))
#'   uid <- "ucc0014"
#'   get_ccdbraw(uid, savetoxlsx = FALSE) |>
#'     set_sub() |>
#'     set_ccsqno(fn = fn) |>
#'     set_celldenbio() |>
#'     set_ccdb(uid = uid)
#' }
#'
set_ccdb <- function(
    ccdbrevised,
    uid,
    conn = cctdb::get_dbconn(paste0(uid, "pub"), TRUE),
    fn = tempfile(pattern = "cctax_fixing_", fileext = ".xlsx")) {
  list(revised = ccdbrevised) |> writexl::write_xlsx(fn)
  system(paste("open", fn))

  readstr <- NA
  while (!(readstr %in% c("done", "stop"))) {
    readstr <- readline(prompt = "请确认excel中revised表中cc数据是否正确\n
      输入`done`将数据上传至数据库!\n
      输入`stop`退出程序!\n>>>")
  }

  if (readstr == "stop") {
    warning("用户退出程序!")
    DBI::dbDisconnect(conn)
    return(invisible(FALSE))
  }

  if (readstr == "done") {
    # 备份数据库并删除cc数据表中的记录
    sqlstr <- paste0(
      "CREATE TABLE `cc_",
      format(lubridate::now(), "%y%m%d%H%M%S"),
      "` AS SELECT * FROM `cc`;"
    )
    DBI::dbExecute(conn, sqlstr)

    sqlstr <- "TRUNCATE TABLE `cc`"
    DBI::dbExecute(conn, sqlstr)

    ccdbrevised <- ccdbrevised |> dplyr::rename(tot = "den")
    DBI::dbWriteTable(conn, "cc", ccdbrevised, append = TRUE, row.names = FALSE)
    DBI::dbDisconnect(conn)

    message("修正后的数据成功更新！")
    return(invisible(TRUE))
  }

  return(invisible(FALSE))
}
