#' Update site data in the drwateR database
#'
#' This function updates the site table in the drwateR database using
#'  information from an Excel file.It first downloads the existing site
#' database to an Excel file, prompts the user to update it,
#' and then applies the changes to the database.
#'
#' @param uid A character string representing the user ID of the cctb database.
#' @param sitefn A character string specifying the path to the Excel file with
#'   updated site information. If NULL, the current site database will be
#' downloaded for the user to update.
#' @param conn A MySQL connection object. If NULL, a new connection will be
#' established using `cctdb::get_dbconn()`.
#' @param dbdisconn A logical indicating whether to disconnect from the database
#' after execution (default is TRUE).
#'
#' @return A logical indicating whether the site table update was successful.
#'
#' @importFrom cctdb get_dbconn
#' @importFrom DBI dbGetQuery dbExecute dbDisconnect
#' @importFrom tibble as_tibble
#' @importFrom dplyr rename mutate select arrange bind_rows slice
#' @importFrom writexl write_xlsx
#' @importFrom readxl read_xlsx
#' @importFrom tidyselect everything
#' @importFrom purrr map_lgl
#' @export
#'
upd_sitedb <- function(
    uid,
    sitefn = NULL,
    conn = cctdb::get_dbconn(gsub("^(.*)$", "\\1pub", uid), TRUE),
    dbdisconn = TRUE) {
  # Define file path for the existing site database
  filepath <- file.path(Sys.getenv("HOME"), "sitefn.xlsx")

  # Download the existing site database and write it to an Excel file
  sitedb <- dplyr::tbl(conn, "site") |>
    dplyr::collect() |>
    dplyr::mutate(oldcode = .data$code) |>
    dplyr::relocate("oldcode")

  writexl::write_xlsx(sitedb, filepath)

  if (is.null(sitefn)) {
    # Notify the user to update the site database and provide instructions
    message(
      "Please update the \"",
      filepath,
      "\", and run me again with `sitefn` specified to continue..."
    )
    return(system(paste0("open ", filepath)))
  }

  # Read updated site information from the specified Excel file
  sitedf <- readxl::read_xlsx(sitefn)
  print(sitedf, n = 1000)

  # Confirm with the user before proceeding
  warnresponse <- readline(prompt = "I have confirmed that sitedf is correct!\n
    Type `YES` to overwrite TABLE site ..\n>>>")
  if (warnresponse != "YES") {
    stop("Quit by user!")
  }

  # Prepare SQL statements for updating the site codes
  sitetobeupdated <- sitedf |>
    dplyr::filter(.data$code != .data$oldcode) |>
    dplyr::select(c("oldcode", "code")) |>
    dplyr::mutate(sqlstr = paste0(
      "UPDATE `cc` SET `site` = '",
      .data$code, "' WHERE `site` = '", .data$oldcode, "';"
    )) |>
    dplyr::mutate(flag = purrr::map_lgl(
      .data$sqlstr,
      ~ DBI::dbExecute(conn, .x)
    ))

  print(sitetobeupdated)

  # Truncate the existing site table
  DBI::dbExecute(conn, "TRUNCATE TABLE `site`;")

  # Write the updated site data to the database
  flag <- sitedf |>
    dplyr::bind_rows(dplyr::slice(sitedb, 0)) |>
    dplyr::select(-.data$oldcode) |>
    dplyr::arrange("belongto2", "belongto1", "code") |>
    DBI::dbWriteTable(
      conn = conn,
      name = "site",
      value = _, # Use `.` to refer to the current data frame in dplyr
      overwrite = FALSE,
      append = TRUE,
      row.names = FALSE
    )

  message(
    "Table `site` was truncated, and the site database was backed up in ",
    filepath
  )

  if (isTRUE(dbdisconn)) {
    DBI::dbDisconnect(conn)
  }

  return(flag)
}
